from .spells import Spell


class CallLightning(Spell):
    """A storm cloud appears in the shape of a cylinder that is 10 feet tall with a 
    60-foot radius, centered on a point you can see 100 feet directly above you. The
     spell fails if you can’t see a point in the air where the storm cloud could 
    appear (for example, if you are in a room that can’t accommodate the cloud). 
    
    
    When you cast the spell, choose a point you can see within range. A bolt of 
    lightning flashes down from the cloud to that point. Each creature within 5 feet
     of that point must make a Dexterity saving throw. A creature takes 3d10 
    lightning damage on a failed save, or half as much damage on a successful one. 
    On each of your turns until the spell ends, you can use your action to call down
     lightning in this way again, targeting the same point or a different one. 
    
    If 
    you are outdoors in stormy conditions when you cast this spell, the spell gives 
    you control over the existing storm instead of creating a new one. Under such 
    conditions, the spell’s damage increases by 1d10.
    
    At Higher Levels: When you 
    cast this spell using a spell slot of 4th or higher level, the damage increases 
    by 1d10 for each slot level above 3rd.
    """
    name = "Call Lightning"
    level = 3
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid',)


class CalmEmotions(Spell):
    """You attempt to suppress strong emotions in a group of people. 
    Each humanoid in 
    a 20-foot-radius sphere centered on a point you choose within range must make a 
    Charisma saving throw; a creature can choose to fail this saving throw if it 
    wishes. If a creature fails its saving throw, choose one of the following two 
    effects. You can suppress any effect causing a target to be charmed or 
    frightened. When this spell ends, any suppressed effect resumes, provided that 
    its duration has not expired in the meantime. 
    
    Alternatively, you can make a 
    target indifferent about creatures of your choice that it is hostile toward. 
    This indifference ends if the target is attacked or harmed by a spell or if it 
    witnesses any of its friends being harmed. When the spell ends, the creature 
    becomes hostile again, unless the DM rules otherwise.
    """
    name = "Calm Emotions"
    level = 2
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Cleric')


class Catapult(Spell):
    """Choose one object weighing 1 to 5 pounds within range that isn’t being worn or 
    carried. The object flies in a straight line up to 90 feet in a direction you 
    choose before falling to the ground, stopping early if it impacts against a 
    solid surface. If the object would strike a creature, that creature must make a 
    Dexterity saving throw. On a failed save, the object strikes the target and 
    stops moving. In either case, both the object and the creature or solid surface 
    take 3d8 bludgeoning damage.
    At Higher Levels. When you cast this spell using a 
    spell slot of 2nd level or higher, the maximum weight of objects that you can 
    target with this spell increases by 5 pounds, and the damage increases by 1d8, 
    for each slot level above 1st.
    """
    name = "Catapult"
    level = 1
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ('S',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Sorcerer', 'Wizard')


class Catnap(Spell):
    """You make a calming gesture, and up to three willing creatures of your choice 
    that you can see within range fall unconscious for the spell’s duration. The 
    spell ends on a target early if it takes damage or someone uses an action to 
    shake or slap it awake. If a target remains unconscious for the full duration, 
    that target gains the benefit of a short rest, and it can’t be affected by this 
    spell again until it finishes a long rest.
    
    At Higher Levels: When you cast this
     spell using a spell slot of 4th level or higher, you can target one additional 
    willing creature for each slot level above 3rd.
    """
    name = "Catnap"
    level = 3
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('S', 'M')
    materials = """A pinch of sand"""
    duration = "10 minutes"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Wizard', 'Bard', 'Sorcerer')


class CauseFear(Spell):
    """You awaken the sense of mortality in one creature you can see within range. A 
    construct or an undead is immune to this effect. The target must succeed on a 
    Wisdom saving throw or become frightened of you until the spell ends. The 
    frightened target can repeat the saving throw at the end of each of its turns, 
    ending the effect on itself on a success.
    
    At Higher Levels: When you cast this 
    spell using a spell slot of 2nd level or higher, you can target one additional 
    creature for each slot level above lst. The creatures must be within 30 feet of 
    each other when you target them.
    """
    name = "Cause Fear"
    level = 1
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Warlock', 'Wizard')


class Ceremony(Spell):
    """You perform a special religious ceremony that is infused with magic. When you 
    cast the spell, choose one of the following rites, the target of which must be 
    within 10 feet of you throughout the casting.
    Atonement. You touch one willing 
    creature whose alignment has changed, and you make a DC 20 Wisdom (Insight) 
    check. On a successful check, you restore the target to its original alignment.
    
    Bless Water. You touch one vial of water and cause it to become holy water.
    
    Coming of Age. You touch one humanoid who is a young adult. For the next 24 
    hours, whenever the target makes an ability check, it can roll a d4 and add the 
    number rolled to the ability check. A creature can benefit from this rite only 
    once.
    Dedication. You touch one humanoid who wishes to be dedicated to your 
    god’s service. For the next 24 hours, whenever the target makes a saving throw, 
    it can roll a d4 and add the number rolled to the save. A creature can benefit 
    from this rite only once.
    Funeral Rite. You touch one corpse, and for the next 7
     days, the target can’t become undead by any means short of a wish spell.
    
    Wedding. You touch adult humanoids willing to be bonded together in marriage. 
    For the next 7 days, each target gains a +2 bonus to AC while they are within 30
     feet of each other. A creature can benefit from this rite again only if 
    widowed.
    """
    name = "Ceremony"
    level = 1
    casting_time = "1 hour"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """25 gp worth of powdered silver,which the spell consumes"""
    duration = "Instantaneous"
    ritual = True
    magic_school = "Abjuration"
    classes = ('Cleric', 'Paladin')


class ChainLightning(Spell):
    """You create a bolt of lightning that arcs toward a target of your choice that you
     can see within range. Three bolts then leap from that target to as many as 
    three other targets, each of which must be within 30 feet of the first target. A
     target can be a creature or an object and can be targeted by only one of the 
    bolts. 
    
    A target must make a Dexterity saving throw. The target takes 10d8 
    lightning damage on a failed save, or half as much on a successful one.
    
    At 
    Higher Levels: When you cast this spell using a spell slot of 7th level or 
    higher, one additional bolt leaps from the first target to another target for 
    each slot level above 6th.
    """
    name = "Chain Lightning"
    level = 6
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ('V', 'S', 'M')
    materials = """A bit of fur; a piece of amber, glass, or a crystal rod; and three silver pins"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Wizard')


class ChaosBolt(Spell):
    """You hurl an undulating, warbling mass of chaotic energy at one creature in 
    range. Make a ranged spell attack against the target. On a hit, the target takes
     2d8 + 1d6 damage. Choose one of the dSs. The number rolled on that die 
    determines the attacks damage type, as shown below.
    d8 / Damage Type
    1 / Acid
    2 
    / Cold
    3 / Fire
    4 / Force
    5 / Lightning
    6 / Poison
    7 / Psychic
    8 / Thunder
    If 
    you roll the same number on both d8s, the chaotic energy leaps from the target 
    to a different creature of your choice within 30 feet of it. Make a new attack 
    roll against the new target, and make a new damage roll, which could cause the 
    chaotic energy to leap again. A creature can be targeted only once by each 
    casting of this spell.
    
    At Higher Levels: When you cast this spell using a spell
     slot of 2nd level or higher, each target takes 1d6 extra damage of the type 
    rolled for each slot level above 1st.
    """
    name = "Chaos Bolt"
    level = 1
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer',)


class CharmMonster(Spell):
    """You attempt to charm a creature you can see within range. It must make a Wisdom 
    saving throw, and it does so with advantage if you or your companions are 
    fighting it. If it fails the saving throw, it is charmed by you until the spell 
    ends or until you or your companions do anything harmful to it. The charmed 
    creature is friendly to you. When the spell ends, the creature knows it was 
    charmed by you.
    
    At Higher Levels: When you cast this spell using a spell slot 
    of 5th level or higher, you can target one additional creature for each slot 
    level above 4th. The creatures must be within 30 feet of each other when you 
    target them.
    """
    name = "Charm Monster"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "1 hour"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Druid', 'Sorcerer', 'Warlock', 'Wizard')


class CharmPerson(Spell):
    """You attempt to charm a humanoid you can see within range. 
    It must make a Wisdom
     saving throw, and does so with advantage if you or your companions are fighting
     it. If it fails the saving throw, it is charmed by you until the spell ends or 
    until you  or your companions do anything harmful to it.The charmed creature 
    regards you as a friendly acquaintance. When the spell ends, the creature knows 
    it was charmed by you.
    
    At Higher Levels: When you cast this spell using a spell
     slot of 2nd level or higher, you can target one additional creature for each 
    slot level above 1st. The creatures must be within 30 feet of each other when 
    you target them.
    """
    name = "Charm Person"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "1 hour"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Druid', 'Sorcerer', 'Warlock', 'Wizard')


class ChillTouch(Spell):
    """You create a ghostly, skeletal hand in the space of a creature within range. 
    
    Make a ranged spell attack against the creature to assail it with the chill of 
    the grave. On a hit, the target takes 1d8 necrotic damage, and it can’t regain 
    hit points until the start of your next turn. Until then, the hand clings to the
     target. If you hit an undead target, it also has disadvantage on attack rolls 
    against you until the end of your next turn.
    
    At Higher Levels: This spell’s 
    damage increases by 1d8 when you reach 5th level (2d8), 11th level (3d8), and 
    17th level (4d8).
    """
    name = "Chill Touch"
    level = 0
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "1 round"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class ChromaticOrb(Spell):
    """You hurl a 4-inch-diameter sphere of energy at a creature that you can see 
    within range. You choose acid, cold, fire, lightning, poison, or thunder for the
     type of orb you create, and then make a ranged spell attack against the target.
     If the attack hits, the creature takes 3d8 damage of the type you chose.
    
    At 
    Higher Levels: When you cast this spell using a spell slot of 2nd level or 
    higher, the damage increases by 1d8 for each slot level above 1st.
    """
    name = "Chromatic Orb"
    level = 1
    casting_time = "1 action"
    casting_range = "90 feet"
    components = ('V', 'S', 'M')
    materials = """A diamond worth at least 50 gp"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Wizard')


class CircleOfDeath(Spell):
    """A sphere of negative energy ripples out in a 60-foot-radius sphere from a point 
    within range. Each creature in that area must make a Constitution saving throw. 
    A target takes 8d6 necrotic damage on a failed save, or half as much damage on a
     successful one.
    
    At Higher Levels: W hen you cast this spell using a spell slot
     of 7th level or higher, the damage increases by 2d6 for each slot level above 
    6th.
    """
    name = "Circle Of Death"
    level = 6
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ('V', 'S', 'M')
    materials = """The powder of a crushed black pearl worth at least 500 gp"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class CircleOfPower(Spell):
    """Divine energy radiates from you, distorting and diffusing magical energy within 
    30 feet of you. 
    Until the spell ends, the sphere moves with you, centered on 
    you. For the duration, each friendly creature in the area (including you) has 
    advantage on saving throws against spells and other magical effects. 
    
    
    Additionally, when an affected creature succeeds on a saving throw made against 
    a spell or magical effect that allows it to make a saving throw to take only 
    half damage, it instead takes no damage if it succeeds on the saving throws.
    """
    name = "Circle Of Power"
    level = 5
    casting_time = "1 action"
    casting_range = "Self (30-foot radius)"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Abjuration"
    classes = ('Paladin',)


class Clairvoyance(Spell):
    """You create an invisible sensor within range in a location familiar to you (a 
    place you have visited or seen before) or in an obvious location that is 
    unfamiliar to you (such as behind a door, around a corner, or in a grove of 
    trees). The sensor remains in place for the duration, and it can’t be attacked 
    or otherwise interacted with. 
    
    When you cast the spell, you choose seeing or 
    hearing. You can use the chosen sense through the sensor as if you were in its 
    space. As your action, you can switch between seeing and hearing. A creature 
    that can see the sensor (such as a creature benefitting from see invisibility or
     truesight) sees a luminous, intangible orb about the size of your fist.
    """
    name = "Clairvoyance"
    level = 3
    casting_time = "10 minutes"
    casting_range = "1 mile"
    components = ('V', 'S', 'M')
    materials = """A focus worth at least 100 gp, either a jeweled horn for hearing or a glass eye for seeing"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Sorcerer', 'Wizard')


class Clone(Spell):
    """This spell grows an inert duplicate of a living creature as a safeguard against 
    death. 
    This clone forms inside a sealed vessel and grows to full size and 
    maturity after 120 days; you can also choose to have the clone be a younger 
    version of the same creature. It remains inert and endures indefinitely, as long
     as its vessel remains undisturbed.
    
    
     At any time after the clone matures, if 
    the original creature dies, its soul transfers to the clone, provided that the 
    soul is free and willing to return. The clone is physically identical to the 
    original and has the same personality, memories, and abilities, but none of the 
    original’s equipment. The original creature’s physical remains, if they still 
    exist, becom e inert and can’t thereafter be restored to life, since the 
    creature’s soul is elsewhere.
    """
    name = "Clone"
    level = 8
    casting_time = "1 hour"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """A diamond worth at least 1,000 gp and at least 1 cubic inch of flesh of the creature that is to be cloned, which the spell consum es, and a vessel worth at least 2,000 gp that has a sealable lid and is large enough to hold a medium creature, such"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Wizard',)


class CloudOfDaggers(Spell):
    """You fill the air with spinning daggers in a cube 5 feet on each side, centered 
    on a point you choose within range. A creature takes 4d4 slashing damage when it
     enters the spell’s area for the first time on a turn or starts its turn there.
    
    
    At Higher Levels: When you cast this spell using a spell slot of 3rd level or 
    higher, the damage increases by 2d4 for each slot level above 2nd.
    """
    name = "Cloud Of Daggers"
    level = 2
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S', 'M')
    materials = """A sliver of glass"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Bard', 'Sorcerer', 'Warlock', 'Wizard')


class Cloudkill(Spell):
    """You create a 20-foot-radius sphere of poisonous, yellow-green fog centered on a 
    point you choose within range. The fog spreads around corners. It lasts for the 
    duration or until strong wind dispereses the fog, ending the spell. Its area is 
    heavily obscured. 
    
    When a creature enters the spell’s area for the first time 
    on a turn or starts its turn there, that creature must make a Constitution 
    saving throw. The creature takes 5d8 poison damageon a failed save, or half as 
    much damage on a successful one. Creatures are affected even if they hold their 
    breath or don’t need to breathe. 
    
    The fog moves 10 feet away from you at the 
    start of each of your turns, rolling along the surface of the ground. The 
    vapors, being heavier than air, sink to the lowest level of the land, even 
    pouring down openings.
    
    At Higher Levels: When you cast this spell using a spell
     slot of 6th level or higher, the damage increases by 1d8 for each slot level 
    above 5th.
    """
    name = "Cloudkill"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Sorcerer', 'Wizard')


class ColorSpray(Spell):
    """A dazzling array of flashing, colored light springs from your hand. 
    Roll 6d10, 
    the total is how many hit points of creatures this spell can effect. Creatures 
    in a 15-foot cone originating from you are affected in ascending order of their 
    current hit points (ignoring unconscious creatures and creatures that can’t 
    see). 
    
    Starting with the creature that has the lowest current hit points, each 
    creature affected by this spell is blinded until the spell ends. Subtract each 
    creature’s hit points from the total before moving on to the creature with the 
    next lowest hit points. A creature’s hit points must be equal to or less than 
    the remaining total for the creature to be affected.
    
    At Higher Levels: When you
     cast this spell using a spell slot of 2nd level or higher, roll an additional 
    2d10 for each slot level above 1st.
    """
    name = "Color Spray"
    level = 1
    casting_time = "1 action"
    casting_range = "Self (15-foot cone)"
    components = ('V', 'S', 'M')
    materials = """"""
    duration = "1 round"
    ritual = False
    magic_school = "Illusion"
    classes = ('Sorcerer', 'Wizard')


class Command(Spell):
    """You speak a one-word command to a creature you can see within range. 
    The target
     must succeed on a Wisdom saving throw or follow the command on its next turn. 
    The spell has no effect if the target is undead, if it doesn’t understand your 
    language, or if your command is directly harmful to it.  Some typical commands 
    and their effects follow. You might issue a command other than one described 
    here. If you do so, the DM determines how the target behaves. If the target 
    can’t follow your command, the spell ends.
    Approach The target moves toward you 
    by the shortest and most direct route, ending its turn if it moves within 5 feet
     of you.
    Drop The target drops whatever it is holding and then ends its turn.
    
    Flee The target spends its turn moving away from you by the fastest available 
    means.
    Grovel The target falls prone and then ends its turn.
    Halt The target 
    doesn’t move and takes no actions. A flying creature stays aloft, provided that 
    it is able to do so. If it must move to stay aloft, it flies the minimum 
    distance needed to remain in the air.
    
    At Higher Levels: When you cast this 
    spell using a spell slot of 2nd level or higher, you can affect one additional 
    creature for each slot level above 1st. The creatures must be within 30 feet of 
    each other when you target them
    """
    name = "Command"
    level = 1
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V',)
    materials = """"""
    duration = "1 round"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Cleric', 'Paladin')


class Commune(Spell):
    """You contact your deity or a divine proxy and ask up to three questions that can 
    be answered with a yes or no. You must ask your questions before the spell ends.
     You receive a correct answer for each question. 
    
    Divine beings aren’t 
    necessarily omniscient, so you might receive “unclear” as an answer if a 
    question pertains to information that lies beyond the deity’s knowledge. In a 
    case where a one-word answer could be misleading or contrary to the deity’s 
    interests, the DM might offer a short phrase as an answer instead. 
    
    If you cast
     the spell two or more times before finishing your next long rest, there is a 
    cumulative 25 percent chance for each casting after the first that you get no 
    answer. The DM makes this roll in secret.
    """
    name = "Commune"
    level = 5
    casting_time = "1 minute"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """Incense and a vial of holy or unholy water"""
    duration = "1 minute"
    ritual = True
    magic_school = "Divination"
    classes = ('Cleric',)


class CommuneWithNature(Spell):
    """You briefly become one with nature and gain knowledge of the surrounding 
    territory. 
    In the outdoors, the spell gives you knowledge of the land within 3 
    miles of you. In caves and other natural underground settings, the radius is 
    limited to 300 feet. The spell doesn’t function where nature has been replaced 
    by construction, such as in dungeons and towns. 
    
    You instantly gain knowledge 
    of up to three facts of your choice about any of the following subjects as they 
    relate to the area: 
    
    •  terrain and bodies of water 
    •  prevalent plants, 
    minerals, animals, or peoples 
    •  powerful celestials, fey, fiends, elementals, 
    or undead 
    •  influence from other planes of existence 
    •  buildings 
    
    For 
    example, you could determine the location of powerful undead in the area, the 
    location of major sources of safe drinking water, and the location of any nearby
     towns.
    """
    name = "Commune With Nature"
    level = 5
    casting_time = "1 minute"
    casting_range = "Self"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = True
    magic_school = "Divination"
    classes = ('Druid', 'Ranger')


class CompelledDuel(Spell):
    """You attempt to compel a creature into a duel. 
    One creature that you can see 
    within range must make a Wisdom saving throw. On a failed save, the creature is 
    drawn to you, compelled by your divine demand. For the duration, it has 
    disadvantage on attack rolls against creatures other than you, and must make a 
    Wisdom saving throw each time it attempts to move to a space that is more than 
    30 feet away from you; if it succeeds on this saving throw, this spell doesn’t 
    restrict the target’s movement for that turn. 
    
    The spell ends if you attack any
     other creature, if you cast a spell that targets a hostile creature other than 
    the target, if a creature friendly to you damages the target or casts a harmful 
    spell on it, or if you end your turn more than 30 feet away from the target.
    """
    name = "Compelled Duel"
    level = 1
    casting_time = "1 bonus action"
    casting_range = "30 feet"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Paladin',)


class ComprehendLanguages(Spell):
    """For the duration, you understand the literal meaning of any spoken language that
     you hear. 
    You also understand any spoken language that you hear. You also 
    understand any written language that you see, but you must be touching the 
    surface of which the words are written. It takes about 1 minute to read one page
     of text. 
    
    This spell doesn’t decode secret messages in a text or glyph, such 
    as an arcane sigil, that isn’t part of a written language.
    """
    name = "Comprehend Languages"
    level = 1
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A pinch of soot and salt"""
    duration = "1 hour"
    ritual = True
    magic_school = "Divination"
    classes = ('Bard', 'Sorcerer', 'Warlock', 'Wizard')


class Compulsion(Spell):
    """Creatures of your choice that you can see within range and that can hear you 
    must make a Wisdom saving throw. 
    A target automatically succeeds on this saving
     throw if it can’t be charmed. On a failed save, a target is affected by this 
    spell. Until the spell ends, you can use a bonus action on each of your turns to
     designate a direction that is horizontal to you. Each affected target must use 
    as much of its movement as possible to move in that direction on its next turn. 
    It can take its action before it moves. After moving in this way, it can make 
    another Wisdom saving throw to try to end the effect. 
    
    A target isn’t compelled
     to move into an obviously deadly hazard, such as a fire pit, but it will 
    provoke opportunity attacks to move in the designated direction.
    """
    name = "Compulsion"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard',)


class ConeOfCold(Spell):
    """A blast of cold air erupts from your hands. 
    Each creature in a 60-foot cone 
    must make a Constitution saving throw. 
    
    A creature takes 8d8 cold damage on a 
    failed save, or half as much damage on a successful one. A creature killed by 
    this spell becomes a frozen statue until it thaws.
    
    At Higher Levels: When you 
    cast this spell using a spell slot of 6th level or higher, the damage increases 
    by 1d8 for each slot level above 5th.
    """
    name = "Cone Of Cold"
    level = 5
    casting_time = "1 action"
    casting_range = "Self (60-foot cone)"
    components = ('V', 'S', 'M')
    materials = """A small crystal or glass cone"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Wizard')


class Confusion(Spell):
    """This spell assaults and twists creatures’ minds, spawning delusions and 
    provoking uncontrolled actions. Each creature in a 10-foot-radius sphere 
    centered on a point you choose within range must succeed on a Wisdom saving 
    throw when you cast this spell or be affected by it. 
    
    An affected target can’t 
    take reactions and must roll a d10 at the start of each of its turns to 
    determine its behavior for that turn. 
    
    d10   Behavior 
    
     1. The creature uses 
    all its movement to move in a random direction. To determine the direction, roll
     a d8 and assign a direction to each die face. The creature doesn’t take an 
    action this turn. 
     
    2-6. The creature doesn’t move or take actions this turn.
     
    
    7-8. The creature uses its action to make a melee attack against a randomly 
    determined creature within its reach. If there is no creature within its reach, 
    the creature does nothing this turn. 
    
     9-10. The creature can act and move 
    normally. 
    
    At the end of its turns, an affected target can make a Wisdom saving
     throw. It it succeeds, this effect ends for that target.
    
    At Higher Levels: 
    When you cast this spell using a spell slot of 5th level or higher, the radius 
    of the sphere increases by 5 feet for each slot level above 4th
    """
    name = "Confusion"
    level = 4
    casting_time = "1 action"
    casting_range = "90 feet"
    components = ('V', 'S', 'M')
    materials = """Three nut shells"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Druid', 'Sorcerer', 'Wizard')


class ConjureAnimals(Spell):
    """You summon fey spirits that take the form of beasts and appear in unoccupied 
    spaces that you can see within range. 
    
    Choose one of the following options for 
    what appears: 
    
    •  One beast of challenge rating 2 or lower 
    •  Two beasts of 
    challenge rating 1  or lower 
    •  Four beasts of challenge rating 1/2 or lower 
    •
      Eight beasts of challenge rating 1/4 or lower 
    
    Each beast is also considered 
    fey, and it disappears when it drops to 0 hit points or when the spell ends. 
    
    
    The summoned creatures are friendly to you and your companions. Roll initiative 
    for the summoned creatures as a group, which has its own turns. They obey any 
    verbal commands that you issue to them (no action required by you). If you don’t
     issue any commands to them, they defend themselves from hostile creatures, but 
    otherwise take no actions. 
    The DM has the creatures’ statistics.
    
    At Higher 
    Levels: When you cast this spell using certain higher-level spell slots, you 
    choose one of the summoning options above, and more creatures appear: 
    
    twice 
    as many with a 5th-level slot
    three times as many with a 7th-level slot 
    four 
    times as many with a 9th-level slot.
    """
    name = "Conjure Animals"
    level = 3
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Ranger')


class ConjureBarrage(Spell):
    """You throw a nonmagical weapon or fire a piece of nonmagical ammunition into the 
    air to create a cone of identical weapons that shoot forward and then disappear.
     Each creature in a 60-foot cone must succeed on a Dexterity saving throw. A 
    creature takes 3d8 damage on a failed save, or half as much damage on a 
    successful one. The damage type is the same as that of the weapon or ammunition 
    used as a component.
    """
    name = "Conjure Barrage"
    level = 3
    casting_time = "1 action"
    casting_range = "Self (60-foot cone)"
    components = ('V', 'S', 'M')
    materials = """One piece of ammunition or a thrown weapon"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Ranger',)


class ConjureCelestial(Spell):
    """You summon a celestial of challenge rating 4 or lower, which appears in an 
    unoccupied space that you can see within range. The celestial disappears when it
     drops to 0 hit points or when the spell ends. 
    
    The celestial is friendly to 
    you and your companions for the duration. Roll initiative for the celestial, 
    which has its own turns. It obeys any verbal commands that you issue to it (no 
    action required by you), as long as they don’t violate its alignment. If you 
    don’t issue any commands to the celestial, it defends itself from hostile 
    creatures but otherwise takes no actions 
    The DM has the celestial’s statistics.
    
    
    At Higher Levels: When you cast this spell using a 9th-level spell slot, you 
    summon a celestial of challenge rating 5 or lower.
    """
    name = "Conjure Celestial"
    level = 7
    casting_time = "1 minute"
    casting_range = "90 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Cleric',)


class ConjureElemental(Spell):
    """You call forth an elemental servant. 
    Choose an area of air, earth, fire, or 
    water that fills a 10-foot cube within range. An elemental of challenge rating 5
     or lower appropriate to the area you chose appears in an unoccupied space 
    within 10 feet of it. For example, a fire elemental emerges from a bonfire, and 
    an earth elemental rises up from the ground. The elemental disappears when it 
    drops to 0 hit points or when the spell ends. 
    
    The elemental is friendly to you
     and your companions for the duration. Roll initiative for the elemental, which 
    has its own turns. It obeys any verbal commands that you issue to it (no action 
    required by you). If you don’t issue any commands to the elemental, it defends 
    itself from hostile creatures but otherwise takes no actions. 
    
    If your 
    concentration is broken, the elemental doesn’t disappear. Instead, you lose 
    control of the elemental, it becom es hostile toward you and your companions, 
    and it might attack. An uncontrolled elemental can’t be dismissed by you, and it
     disappears 1 hour after you summoned it. The DM has the elemental’s statistics.
    
    
    At Higher Levels: When you cast this spell using a spell slot of 6th level or 
    higher, the challenge rating increases by 1 for each slot level above 5th.
    """
    name = "Conjure Elemental"
    level = 5
    casting_time = "1 minute"
    casting_range = "90 feet"
    components = ('V', 'S', 'M')
    materials = """Burning incense for air, soft clay for earth, sulfur and phosphorus for fire, or water and sand for water"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Wizard')


class ConjureFey(Spell):
    """You summon a fey creature of challenge rating 6 or lower, or a fey spirit that 
    takes the form of a beast of challenge rating 6 or lower. 
    It appears in an 
    unoccupied space that you can see within range. The fey creature disappears when
     it drops to 0 hit points or when the spell ends. 
    
    The fey creature is friendly
     to you and your companions for the duration. Roll initiative for the creature, 
    which has its own turns. It obeys any verbal commands that you issue to it (no 
    action required by you), as long as they don’t violate its alignment. If you 
    don’t issue any commands to the fey creature, it defends itself from hostile 
    creatures but otherwise takes no actions. 
    
    If your concentration is broken, the
     fey creature doesn’t disappear. Instead, you lose control of the fey creature, 
    it becomes hostile toward you and your companions, and it might attack. An 
    uncontrolled fey creature can’t be dismissed by you, and it disappears 1 hour 
    after you summoned it. 
    The DM has the fey creature’s statistics.
    
    At Higher 
    Levels: When you cast this spell using a spell slot of 7th level or higher, the 
    challenge rating increases by 1 for each slot level above 6th
    """
    name = "Conjure Fey"
    level = 6
    casting_time = "1 minute"
    casting_range = "90 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Warlock')


class ConjureMinorElementals(Spell):
    """You summon elementals that appear in unoccupied spaces that you can see within 
    range.
     You choose one the following options for what appears: 
    -One elemental 
    of challenge rating 2 or lower 
    -Two elementals of challenge rating 1 or lower 
    
    -Four elementals of challenge rating 1/2 or lower 
    -Eight elementals of 
    challenge rating 1/4 or lower. 
    
    An elemental summoned by this spell disappears 
    when it drops to 0 hit points or when the spell ends. 
    
    The summoned creatures 
    are friendly to you and your companions. Roll initiative for the summoned 
    creatures as a group, which has its own turns. They obey any verbal commands 
    that you issue to them (no action required by you). If you don’t issue any 
    commands to them, they defend themselves from hostile creatures, but otherwise 
    take no actions. 
    The DM has the creatures’ statistics.
    
    At Higher Levels: When 
    you cast this spell using certain higher-level spell slots, you choose one of 
    the summoning options above, and more creatures appear: twice as many with a 
    6th-level slot and three times as many with an 8th-level slot.
    """
    name = "Conjure Minor Elementals"
    level = 4
    casting_time = "1 minute"
    casting_range = "90 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Wizard')


class ConjureVolley(Spell):
    """You fire a piece of nonmagical ammunition from a ranged weapon or throw a 
    nonmagical weapon into the air and choose a point within range. 
    Hundreds of 
    duplicates of the ammunition or weapon fall in a volley from above and then 
    disappear. Each creature in a 40-foot-radius. 20-foot-high cylinder centered on 
    that point must make a Dexterity saving throw. A creature takes 8d8 damage on a 
    failed save, or half as much damage on a successful one. The damage type is the 
    same as that of the ammunition or weapon.
    """
    name = "Conjure Volley"
    level = 5
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ('V', 'S', 'M')
    materials = """One piece of ammunition or one thrown weapon"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Ranger',)


class ConjureWoodlandBeings(Spell):
    """You summon fey creatures that appear in unoccupied spaces that you can see 
    within range. 
    
    Choose one of the following options for what appears: 
     •  One 
    fey creature of challenge rating 2 or lower 
     •  Two fey creatures of challenge 
    rating 1 or lower 
     •  Four fey creatures of challenge rating 1/2 or lower 
     •  
    Eight fey creatures of challenge rating 1/4 or lower 
    
    A summoned creature 
    disappears when it drops to 0 hit points or when the spell ends. 
    
    The summoned 
    creatures are friendly to you and your companions. Roll initiative for the 
    summoned creatures as a group, which have their own turns. They obey any verbal 
    commands that you issue to them (no action required by you). If you don’t issue 
    any commands to them, they defend themselves from hostile creatures, but 
    otherwise take no actions. 
    The DM has the creatures’ statistics.
    
    At Higher 
    Levels: When you cast this spell using certain higher-level spell slots, you 
    choose one of the summoning options above, and more creatures appear: 
    twice as
     many with a 6th-level slot
    three times as many with an 8th-level slot.
    """
    name = "Conjure Woodland Beings"
    level = 4
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S', 'M')
    materials = """One holly berry per creature summoned"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Ranger')


class ContactOtherPlane(Spell):
    """You mentally contact a demigod, the spirit of a long-dead sage, or some other 
    mysterious entity from another plane. 
    Contacting this extraplanar intelligence 
    can strain or even break your mind. When you cast this spell, make a DC 15 
    Intelligence saving throw. On a failure, you take 6d6 psychic damage and are 
    insane until you finish a long rest. While insane, you can’t take actions, can’t
     understand what other creatures say, can’t read, and speak only in gibberish. A
     greater restoration spell cast on you ends this effect. 
    
    On a successful save,
     you can ask the entity up to five questions. You must ask your questions before
     the spell ends. The DM answers each question with one word, such as yes, 
    no, maybe, never, irrelevant, or unclear (if the entity doesn’t know 
    the answer to the question). If a one-word answer would be misleading, the DM 
    might instead offer a short phrase as an answer.
    """
    name = "Contact Other Plane"
    level = 5
    casting_time = "1 minute"
    casting_range = "Self"
    components = ('V',)
    materials = """"""
    duration = "1 minute"
    ritual = True
    magic_school = "Divination"
    classes = ('Warlock', 'Wizard')


class Contagion(Spell):
    """Your touch inflicts disease. 
    Make a melee spell attack against a creature 
    within your reach. On a hit, you afflict the creature with a disease of your 
    choice from any of the ones described below. 
    
    At the end of each of the 
    target’s turns, it must make a Constitution saving throw. After failing three of
     these saving throws, the disease’s effects last for the duration, and the 
    creature stops making these saves. After succeeding on three of these saving 
    throws, the creature recovers from the disease, and the spell ends. 
    
    Since this
     spell induces a natural disease in its target, any effect that removes a 
    disease or otherwise ameliorates a disease’s effects apply to it. 
    
    Blinding 
    Sickness 
    Pain grips the creature’s mind, and its eyes turn milky white. The 
    creature has disadvantage on Wisdom checks and Wisdom saving throws and is 
    blinded. 
    
    Filth Fever 
    A raging fever sweeps through the creature’s body. The 
    creature has disadvantage on Strength checks, Strength saving throws, and attack
     rolls that use Strength. 
    
    Flesh Rot
    The creature’s flesh decays. The creature 
    has disadvantage on Charisma checks and vulnerability to all damage. 
    
    Mindfire
     
    The creature’s mind becomes feverish. The creature has disadvantage on 
    Intelligence checks and Intelligence saving throws, and the creature behaves as 
    if under the effects of the confusion spell during combat. 
    
    Seizure 
    The 
    creature is overcome with shaking. The creature has disadvantage on Dexterity 
    checks, Dexterity saving throws, and attack rolls that use Dexterity. 
    
    Slimy 
    Doom 
    The creature begins to bleed uncontrollably. The creature has disadvantage
     on Constitution checks and Constitution saving throws. In addition, whenever 
    the creature takes damage, it is stunned until the end of its next turn.
    """
    name = "Contagion"
    level = 5
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S')
    materials = """"""
    duration = "7 days"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Cleric', 'Druid')


class Contingency(Spell):
    """Choose a spell of 5th level or lower that you can cast, that has a casting time 
    of 1 action, and that can target you. 
    You cast that spell called the 
    contingent spell as part of casting contingency, expending spell slots for 
    both, but the contingent spell doesn’t come into effect. Instead, it takes 
    effect when a certain circumstance occurs. You describe that circumstance when 
    you cast the two spells. For example, a contingency cast with water breathing 
    might stipulate that water breathing comes into effect when you are engulfed in 
    water or a similar liquid. 
    
    The contingent spell takes effect immediately after
     the circumstance is met for the first time, whether or not you want it to. and 
    then contingency ends. 
    
    The contingent spell takes effect only on you, even if 
    it can normally target others. You can use only one contingency spell at a time.
     If you cast this spell again, the effect of another contingency spell on you 
    ends. Also, contingency ends on you if its material component is ever not on 
    your person.
    """
    name = "Contingency"
    level = 6
    casting_time = "10 minutes"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A statuette of yourself carved from ivory and decorated with gems worth at least 1,500 gp"""
    duration = "10 days"
    ritual = False
    magic_school = "Evocation"
    classes = ('Wizard',)


class ContinualFlame(Spell):
    """A flame, equivalent in brightness to a torch, springs forth from an object that 
    you touch. 
    The effect looks like a regular flame, but it creates no heat and 
    doesn’t use oxygen. A continual flame can be covered or hidden but not smothered
     or quenched.
    """
    name = "Continual Flame"
    level = 2
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """Ruby dust worth 50 gp, which the spell consumes"""
    duration = "Until dispelled"
    ritual = False
    magic_school = "Evocation"
    classes = ('Cleric', 'Wizard')


class ControlFlames(Spell):
    """You choose a nonmagical flame that you can see within range and that fits within
     a 5-foot cube. You affect it in one of the following ways:
    - You 
    instantaneously expand the flame 5 feet in one direction, provided that wood or 
    other fuel is present in the new location.
    - You instantaneously extinguish the 
    flames within the cube.
    - You double or halve the area of bright light and dim 
    light cast by the flame, change its color, or both. The change lasts for 1 hour.
    
    - You cause simple shapes — such as the vague form of a creature, an inanimate 
    object, or a location — to appear within the flames and animate as you like. The
     shapes last for 1 hour.
    If you cast this spell multiple times, you can have up 
    to three non-instantaneous
    """
    name = "Control Flames"
    level = 0
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('S',)
    materials = """"""
    duration = "Instantaneous or 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Druid', 'Sorcerer', 'Wizard')


class ControlWater(Spell):
    """Until the spell ends, you control any freestanding water inside an area you 
    choose that is a cube up to 100 feet on a side. 
    You can choose from any of the 
    following effects when you cast this spell. As an action on your turn, you can 
    repeat the same effect or choose a different one. 
    
    Flood 
    You cause the water 
    level of all standing water in the area to rise by as much as 20 feet. If the 
    area includes a shore, the flooding water spills over onto dry land. If you 
    choose an area in a large body of water, you instead create a 20-foot tall wave 
    that travels from one side of the area to the other and then crashes down. Any 
    Huge or smaller vehicles in the wave’s path are carried with it to the other 
    side. Any Huge or smaller vehicles struck by the wave have a 25 percent chance 
    of capsizing. The water level remains elevated until the spell ends or you 
    choose a different effect. If this effect produced a wave, the wave repeats on 
    the start of your next turn while the flood effect lasts. 
    
    Part Water 
    You 
    cause water in the area to move apart and create a trench. The trench extends 
    across the spell’s area, and the separated water forms a wall to either side. 
    The trench remains until the spell ends or you choose a different effect. The 
    water then slowly fills in the trench over the course of the next round until 
    the normal water level is restored. 
    
    Redirect Flow 
    You cause flowing water in 
    the area to move in a direction you choose, even if the water has to flow over 
    obstacles, up walls, or in other unlikely directions. The water in the area 
    moves as you direct it, but once it moves beyond the spell’s area, it resumes 
    its flow based on the terrain conditions. The water continues to move in the 
    direction you chose until the spell ends or you choose a different effect. 
    
    
    Whirlpool 
    This effect requires a body of water at least 50 feet square and 25 
    feet deep. You cause a whirlpool to form in the center of the area. The 
    whirlpool forms a vortex that is 5 feet wide at the base, up to 50 feet wide at 
    the top, and 25 feet tall. Any creature or object in the water and within 25 
    feet of the vortex is pulled 10 feet toward it. A creature can swim away from 
    the vortex by making a Strength (Athletics) check against your spell save DC. 
    
    When a creature enters the vortex for the first time on a turn or starts its 
    turn there, it must make a Strength saving throw. On a failed save, the creature
     takes 2d8 bludgeoning damage and is caught in the vortex until the spell ends. 
    On a successful save, the creature takes half damage, and isn’t caught in the 
    vortex. A creature caught in the vortex can use its action to try to swim away 
    from the vortex as described above, but has disadvantage on the Strength 
    (Athletics) check to do so. 
    The first time each turn that an object enters the 
    vortex, the object takes 2d8 bludgeoning damage, this damage occurs each round 
    it remains in the vortex.
    """
    name = "Control Water"
    level = 4
    casting_time = "1 action"
    casting_range = "300 feet"
    components = ('V', 'S', 'M')
    materials = """A drop of water and a pinch of dust"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Cleric', 'Druid', 'Wizard')


class ControlWeather(Spell):
    """You take control of the weather within 5 miles of you for the duration. 
    You 
    must be outdoors to cast this spell. Moving to a place where you don’t have a 
    clear path to the sky ends the spell early. 
    
    When you cast the spell, you 
    change the current weather conditions, which are determined by the DM based on 
    the climate and season. You can change precipitation, temperature, and wind. It 
    takes 1d4 x 10 minutes for the new conditions to take effect. Once they do so, 
    you can change the conditions again. When the spell ends, the weather gradually 
    returns to normal. 
    
    When you change the weather conditions, find a current 
    condition on the following tables and change its stage by one, up or down. When 
    changing the wind, you can change its direction. 
    
    Precipitation 
    Stage 1 – 
    Clear, 
    Stage 2 – Light clouds, 
    Stage 3 – Overcast or ground fog, 
    Stage 4 – 
    Rain, hail or snow, 
    Stage 5 – Torrential rain, driving hail or blizzard 
    
    
    Temperature 
    Stage 1 – Unbearable heat, 
    Stage 2 – Hot, 
    Stage 3 – Warm, 
    Stage 
    4 – Cool, 
    Stage 5 – Cold, 
    Stage 6 – Arctic cold 
    
    Wind 
    Stage 1 – Calm, 
    Stage
     2 – Moderate wind, 
    Stage 3 – Strong wind, 
    Stage 4 – Gale, 
    Stage 5 – Storm
    """
    name = "Control Weather"
    level = 8
    casting_time = "10 minutes"
    casting_range = "Self (5-mile radius)"
    components = ('V', 'S', 'M')
    materials = """"""
    duration = "Concentration, up to 8 hours"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Cleric', 'Druid', 'Wizard')


class ControlWinds(Spell):
    """You take control of the air in a 100-foot cube that you can see within range. 
    Choose one of the following effects when you cast the spell. The effect lasts 
    for the spell’s duration, unless you use your action on a later turn to switch 
    to a different effect. You can also use your action to temporarily halt the 
    effect or to restart one you’ve halted.
    Gusts. A wind picks up within the cube, 
    continually blowing in a horizontal direction that you choose. You choose the 
    intensity of the wind: calm, moderate, or strong. If the wind is moderate or 
    strong, ranged weapon attacks that pass through it or that are made against 
    targets within the cube have disadvantage on their attack rolls. If the wind is 
    strong, any creature moving against the wind must spend 1 extra foot of movement
     for each foot moved.
    
    Downdraft. You cause a sustained blast of strong wind to 
    blow downward from the top of the cube. Ranged weapon attacks that pass through 
    the cube
    or that are made against targets within it have disadvantage on their 
    attack rolls. A creature must make a Strength saving throw if it flies into the 
    cube for the first time on a turn or starts its turn there flying. On a failed 
    save, the creature is knocked prone.
    
    Updraft. You cause a sustained updraft 
    within the cube, rising upward from the cube’s bottom edge. Creatures that end a
     fall within the cube take only half damage from the fall. When a creature in 
    the cube makes a vertical jump, the creature can jump up to 10 feet higher than 
    normal.
    """
    name = "Control Winds"
    level = 5
    casting_time = "1 action"
    casting_range = "300 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Druid', 'Sorcerer', 'Wizard')


class CordonOfArrows(Spell):
    """You plant four pieces of nonmagical ammunition – arrows or crossbow bolts – in 
    the ground within range and lay magic upon them to protect an area. 
    Until the 
    spell ends, whenever a creature other than you comes within 30 feet of the 
    ammunition for the first time on a turn or ends its turn there, one piece of 
    ammunition flies up to strike it. The creature must succeed on a Dexterity 
    saving throw or take 1d6 piercing damage. The piece of ammunition is then 
    destroyed. The spell ends when no ammunition remains. 
    
    When you cast this 
    spell, you can designate any creatures you choose, and the spell ignores them.
    
    
    At Higher Levels: When you cast this spell using a spell slot of 3rd level or 
    higher, the amount of ammunition that can be affected increases by two for each 
    slot level above 2nd.
    """
    name = "Cordon Of Arrows"
    level = 2
    casting_time = "1 action"
    casting_range = "5 feet"
    components = ('V', 'S', 'M')
    materials = """Four or more arrows or bolts"""
    duration = "8 hours"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Ranger',)


class Counterspell(Spell):
    """1 reaction, which you take when you see a creature within 60 feet of you casting
     a spell
    
    You attempt to interrupt a creature in the process of casting a spell.
     If the creature is casting a spell of 3rd level or lower, its spell fails and 
    has no effect. If it is casting a spell of 4th level or higher, make an ability 
    check using your spellcasting ability.  The DC equals 10+ the spell’s level. On 
    a success, the creature’s spell fails and has no effect.
    
    At Higher Levels: When
     you cast this spell using a spell slot of 4th level or higher, the interrupted 
    spell has no effect if its level is less than or equal to the level of the spell
     slot you used.
    """
    name = "Counterspell"
    level = 3
    casting_time = "Special"
    casting_range = "60 feet"
    components = ('S',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Abjuration"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class CreateBonfire(Spell):
    """You create a bonfire on ground that you can see within range. Until the spell 
    ends, the magic bonfire fills a 5-foot cube. Any creature in the bonfire’s space
     when you cast the spell must succeed on a Dexterity saving throw or take 1d8 
    fire damage. A creature must also make the saving throw when it moves into the 
    bonfire’s space for the first time on a turn or ends its turn there.
    The bonfire
     ignites flammable objects in its area that aren’t being worn or carried.
    The 
    spell’s damage increases by 1d8 when you reach 5th level (2d8), 11th level 
    (3d8), and 17th level (4d8).
    """
    name = "Create Bonfire"
    level = 0
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Sorcerer', 'Warlock', 'Wizard')


class CreateFoodAndWater(Spell):
    """You create 45 pounds of food and 30 gallons of water on the ground or in 
    containers within range, enough to sustain up to fifteen humanoids or five 
    steeds for 24 hours. The food is bland but nourishing, and spoils if uneaten 
    after 24 hours. The water is clean and doesn’t go bad.
    """
    name = "Create Food And Water"
    level = 3
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Cleric', 'Paladin')


class CreateHomunculus(Spell):
    """While speaking an intricate incantation, you cut yourself with a jewel-encrusted
     dagger, taking 2d4 piercing damage that can’t be reduced in any way. You then 
    drip your blood on the spell’s other components and touch them, transforming 
    them into a special construct called a homunculus. The statistics of the 
    homunculus are in the Monster Manual. It is your faithful companion, and it dies
     if you die. Whenever you finish a long rest, you can spend up to half your Hit 
    Dice if the homunculus is on the same plane of existence as you. When you do so,
     roll each die and add your Constitution modifier to it. Your hit point maximum 
    is reduced by the total, and the homunculus’s hit point maximum and current hit 
    points are
    both increased by it. This process can reduce you to no lower than 1 
    hit point. and the change to your and the homunculus’s hit points ends when you 
    finish your next long rest. The reduction to your hit point maximum can’t be 
    removed by any means before then, except by the homunculus‘s death. You can have
     only one homunculus at a time. If you cast this spell while your homunculus 
    lives, the spell fails.
    """
    name = "Create Homunculus"
    level = 6
    casting_time = "1 hour"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """Clay, ash, and mandrake root, all of which the spell consumes, and a jewel-encrusted dagger worth at least 1,000 gp"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Wizard',)


class CreateOrDestroyWater(Spell):
    """You either create or destroy water. 
    
    Create Water 
    You create up to 10 gallons 
    of clean water within range in an open container. Alternatively, the water falls
     as rain in a 30-foot cube within range, extinguishing exposed flames in the 
    area. 
    
    Destroy Water 
    You destroy up to 10 gallons of water in an open 
    container within range. Alternatively, you destroy fog in a 30-foot cube within 
    range.
    
    At Higher Levels: When you cast this spell using a spell slot of 2nd 
    level or higher, you create or destroy 10 additional gallons of water, or the 
    size of the cube increases by 5 feet, for each slot level above 1st.
    """
    name = "Create Or Destroy Water"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """A drop of water if creating water or a few grains of sand if destroying it"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Cleric', 'Druid')


class CreateUndead(Spell):
    """You can cast this spell only at night. Choose up to three corpses of Medium or 
    Small humanoids within range. Each corpse becomes a ghoul under your control. 
    (The DM has game statistics for these creatures.) 
    
    As a bonus action on each of
     your turns, you can mentally command any creature you animated with this spell 
    if the creature is within 120 feet of you (if you control multiple creatures, 
    you can command any or all of them at the same time, issuing the same command to
     each one). You decide what action the creature will take and where it will move
     during its next turn, or you can issue a general command, such as to guard a 
    particular chamber or corridor. If you issue no commands, the creature only 
    defends itself against hostile creatures. Once given an order, the creature 
    continues to follow it until its task is complete. 
    
    The creature is under your 
    control for 24 hours, after which it stops obeying any command you have given 
    it. To maintain control of the creature for another 24 hours, you must cast this
     spell on the creature before the current 24-hour period ends. This use of the 
    spell reasserts your control over up to three creatures you have animated with 
    this spell, rather than animating new ones.
    
    At Higher Levels: When you cast 
    this spell using a 7th-level spell slot, you can animate or reassert control 
    over four ghouls. 
    When you cast this spell using an 8th-level spell slot, you 
    can animate or reassert control over five ghouls or two ghasts or wights. 
    When
     you cast this spell using a 9th-level spell slot, you can animate or reassert 
    control over six ghouls, three ghasts or wights, or two mummies.
    """
    name = "Create Undead"
    level = 6
    casting_time = "1 minute"
    casting_range = "10 feet"
    components = ('V', 'S', 'M')
    materials = """One clay pot filled with grave dirt, one clay pot filled with brackish water, and one 150 gp black onyx stone for each corpse"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Cleric', 'Warlock', 'Wizard')


class Creation(Spell):
    """You pull wisps of shadow material from the Shadowfell to create a nonliving 
    object of vegetable matter within range: soft goods, rope, wood, or something 
    similar. You can also use this spell to create mineral objects such as stone, 
    crystal, or metal. The object created must be no larger than a 5-foot cube, and 
    the object must be of a form and material that you have seen before. 
    
    The 
    duration depends on the object’s material. If the object is composed of multiple
     materials, use the shortest duration. 
    
    Material — Duration 
    Vegetable matter —
     1 day 
    Stone/crystal — 12 hours 
    Precious metals — 1 hour 
    Gems — 10 minutes 
    
    Adamantine/Mithral — 1 minute 
    
    Using any material created by this spell as 
    another spell’s material component causes that spell to fail.
    
    At Higher Levels:
     When you cast this spell using a spell slot of 6th level or higher, the cube 
    increases by 5 feet for each slot level above 5th.
    """
    name = "Creation"
    level = 5
    casting_time = "1 minute"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """A tiny piece of matter of the same type of the item you plan to create"""
    duration = "Special"
    ritual = False
    magic_school = "Illusion"
    classes = ('Sorcerer', 'Wizard')


class CrownOfMadness(Spell):
    """One humanoid of your choice that you can see within range must succeed on a 
    Wisdom saving throw or become charmed by you for the duration. 
    While the target
     is charmed in this way, a twisted crown of jagged iron appears on its head, and
     a madness glows in its eyes. 
    
    The charmed target must use its action before 
    moving on each of its turns to make a melee attack against a creature other than
     itself that you mentally choose. The target can act normally on its turn if you
     choose no creature or if none are within its reach. 
    
    On your subsequent turns,
     you must use your action to maintain control over the target, or the spell 
    ends. Also, the target can make a Wisdom saving throw at the end of each of its 
    turns. On a success, the spell ends.
    """
    name = "Crown Of Madness"
    level = 2
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Sorcerer', 'Warlock', 'Wizard')


class CrownOfStars(Spell):
    """Seven star-like motes of light appear and orbit your head until the spell ends. 
    You can use a bonus action to send one of the motes streaking toward one 
    creature or object within 120 feet of you. When you do so, make a ranged spell 
    attack. On a hit. the target takes 4d12 radiant damage. Whether you hit or miss,
     the mote is expended. The spell ends early if you expend the last mote. If you 
    have four or more motes remaining, they shed bright light in a 30-foot radius 
    and dim light for an additional 30 feet. Ifyou have one to three motes 
    remaining, they shed dim light in a 30—foot radius.
    
    At Higher Levels: When you 
    cast this spell using a spell slot of 8th level or higher, the number of motes 
    created increases by two for each slot level above 7th.
    """
    name = "Crown Of Stars"
    level = 7
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S')
    materials = """"""
    duration = "1 hour"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class CrusadersMantle(Spell):
    """Holy power radiates from you in an aura with a 30-foot radius, awakening 
    boldness in friendly creatures. Until the spell ends, the aura moves with you, 
    centered on you. While in the aura, each nonhostile creature in the aura 
    (including you) deals an extra 1d4 radiant damage when it hits with a weapon 
    attack.
    """
    name = "Crusaders Mantle"
    level = 3
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ('Paladin',)


class CureWounds(Spell):
    """A creature you touch regains a number of hit points equal to 1d8 + your 
    spellcasting ability modifier. This spell has no effect on undead or constructs.
    
    
    At Higher Levels: When you cast this spell using a spell slot of 2nd level or 
    higher, the healing increases by 1d8 for each slot level above 1st.
    """
    name = "Cure Wounds"
    level = 1
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Bard', 'Cleric', 'Druid', 'Paladin', 'Ranger')





__all__ = ('__version__', 'Character', 'weapons', 'features',
           'character', 'race', 'background', 'spells')

from . import weapons, features, race, background, spells
from .character import Character

import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


__version__ = read('../VERSION')



from .spells import Spell


class LegendLore(Spell):
    """Name or describe a person, place, or object. The spell brings to your mind a 
    brief summary of the significant lore about the thing you named. The lore might 
    consist of current tales, forgotten stories, or even secret lore that has never 
    been widely known. If the thing you named isn’t of legendary importance, you 
    gain no information. The more information you already have about the thing, the 
    more precise and detailed the information you receive is.
    
    The information you 
    learn is accurate but might be couched in figurative language. For example, if 
    you have a mysterious magic axe on hand, the spell might yield this information:
     Woe to the evildoer whose hand touches the axe, for even the haft slices the 
    hand of the evil ones. Only a true Child of Stone, lover and beloved of Moradin,
     may awaken the true powers of the axe, and only with the sacred word Rudnogg on
     the lips.
    """
    name = "Legend Lore"
    level = 5
    casting_time = "10 minutes"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """Incense worth at least 250 gp, which the spell consumes, and four ivory strips worth at least 50 gp each"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Wizard')


class LeomundsSecretChest(Spell):
    """You hide a chest, and all its contents, on the Ethereal Plane.
    You must touch 
    the chest and the miniature replica that serves as a material component for the 
    spell. The chest can contain up to 12 cubic feet of nonliving material (3 feet 
    by 2 feet by 2 feet).
    
    While the chest remains on the Ethereal Plane, you can 
    use an action and touch the replica to recall the chest. It appears in an 
    unoccupied space on the ground within 5 feet of you. You can send the chest back
     to the Ethereal Plane by using an action and touching both the chest and the 
    replica.
    
    After 60 days, there is a cumulative 5 percent chance per day that the
     spell’s effect ends. This effect ends if you cast this spell again, if the 
    smaller replica chest is destroyed, or if you choose to end the spell as an 
    action. If the spell ends and the larger chest is on the Ethereal Plane, it is 
    irretrievably lost.
    """
    name = "Leomunds Secret Chest"
    level = 4
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """An exquisite chest, 3 feet by 2 feet by 2 feet, constructed from rare materials worth at least 5,000 gp, and a tiny replica made from the same materials worth at least 50 gp"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Wizard',)


class LeomundsTinyHut(Spell):
    """A 10-foot-radius immobile dome of force springs into existence around and above 
    you and remains stationary for the duration. The spell ends if you leave its 
    area.
    
    Nine creatures of Medium size or smaller can fit inside the dome with 
    you. The spell fails if its area includes a larger creature or more than nine 
    creatures. Creatures and objects within the dome when you cast this spell can 
    move through it freely. All other creatures and objects are barred from passing 
    through it. Spells and other magical effects can’t extend through the dome or be
     cast through it. The atmosphere inside the space is comfortable and dry, 
    regardless of the weather outside.
    
    Until the spell ends, you can command the 
    interior to become dimly lit or dark. The dome is opaque from the outside, of 
    any color you choose, but it is transparent from the inside.
    """
    name = "Leomunds Tiny Hut"
    level = 3
    casting_time = "1 minute"
    casting_range = "Self (10-foot-radius hemisphere)"
    components = ('V', 'S', 'M')
    materials = """A small crystal bead"""
    duration = "8 hours"
    ritual = True
    magic_school = "Evocation"
    classes = ('Bard', 'Wizard')


class LesserRestoration(Spell):
    """You touch a creature and can end either one disease or one condition afflicting 
    it. The condition can be blinded, deafened, paralyzed, or poisoned.
    """
    name = "Lesser Restoration"
    level = 2
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Abjuration"
    classes = ('Bard', 'Cleric', 'Druid', 'Paladin', 'Ranger')


class Levitate(Spell):
    """One creature or object of your choice that you can see within range rises 
    vertically, up to 20 feet, and remains suspended there for the duration. The 
    spell can levitate a target that weighs up to 500 pounds. An unwilling creature 
    that succeeds on a Constitution saving throw is unaffected.
    
    The target can move
     only by pushing or pulling against a fixed object or surface within reach (such
     as a wall or a ceiling), which allows it to move as if it were climbing. You 
    can change the target’s altitude by up to 20 feet in either direction on your 
    turn. If you are the target, you can move up or down as part of your move. 
    Otherwise, you can use your action to move the target, which must remain within 
    the spell’s range.
    
    When the spell ends, the target floats gently to the ground 
    if it is still aloft.
    """
    name = "Levitate"
    level = 2
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S', 'M')
    materials = """Either a small leather loop or a piece of golden wire bent into a cup shape with a long shank on one end"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Sorcerer', 'Wizard')


class LifeTransference(Spell):
    """You sacrifice some of your health to mend another creature’s injuries. You take 
    4d8 necrotic damage, and one creature of your choice that you can see within 
    range regains a number of hit points equal to twice the necrotic damage you 
    take.
    
    At Higher Levels: When you cast this spell using a spell slot of 4th 
    level or higher, the damage increases by 1d8 for each slot level above 3rd.
    """
    name = "Life Transference"
    level = 3
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Cleric', 'Wizard')


class Light(Spell):
    """You touch one object that is no larger than 10 feet in any dimension. Until the 
    spell ends, the object sheds bright light in a 20-foot radius and dim light for 
    an additional 20 feet. The light can be colored as you like. Completely covering
     the object with something opaque blocks the light. The spell ends if you cast 
    it again or dismiss it as an action.
    
    If you target an object held or worn by a 
    hostile creature, that creature must succeed on a Dexterity saving throw to 
    avoid the spell.
    """
    name = "Light"
    level = 0
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'M')
    materials = """A firefly or phosphorescent moss"""
    duration = "1 hour"
    ritual = False
    magic_school = "Evocation"
    classes = ('Bard', 'Cleric', 'Sorcerer', 'Wizard')


class LightningArrow(Spell):
    """The next time you make a ranged weapon attack during the spell’s duration, the 
    weapon’s ammunition, or the weapon itself if it’s a thrown weapon, transforms 
    into a bolt of lightning. Make the attack roll as normal, The target takes 4d8 
    lightning damage on a hit, or half as much damage on a miss, instead of the 
    weapon’s normal damage.
    
    Whether you hit or miss, each creature within 10 feet 
    of the target must make a Dexterity saving throw. Each of these creatures takes 
    2d8 lightning damage on a failed save, or half as much damage on a successful 
    one.
    
    The piece of ammunition or weapon then returns to its normal form.
    
    At 
    Higher Levels: When you cast this spell using a spell slot of 4th level or 
    higher, the damage for both effects of the spell increases by 1d8 for each slot 
    level above 3rd.
    """
    name = "Lightning Arrow"
    level = 3
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Ranger',)


class LightningBolt(Spell):
    """A stroke of lightning forming a line of 100 feet long and 5 feet wide blasts out
     from you in a direction you choose. Each creature in the line must make a 
    Dexterity saving throw. A creature takes 8d6 lightning damage on a failed save, 
    or half as much damage on a successful one.
    
    The lightning ignites flammable 
    objects in the area that aren’t being worn or carried.
    
    At Higher Levels: When 
    you cast this spell using a spell slot of 4th level or higher, the damage 
    increases by 1d6 for each slot level above 3rd.
    """
    name = "Lightning Bolt"
    level = 3
    casting_time = "1 action"
    casting_range = "Self (100-foot line)"
    components = ('V', 'S', 'M')
    materials = """A bit of fur and a rod of amber, crystal, or glass"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Wizard')


class LightningLure(Spell):
    """You create a lash of lightning energy that strikes at one creature of your 
    choice that you can see within range.
    The target must succeed on a Strength 
    saving throw or be pulled up to 10 feet in a straight line toward you and then 
    take 1d8 lightning damage if it is within 5 feet of you.
    
    At Higher Levels: This
     spell's damage increases by 1d8 when you reach 5th level (2d8), 11th level 
    (3d8), and 17th level (4d8).
    """
    name = "Lightning Lure"
    level = 0
    casting_time = "1 action"
    casting_range = "15 feet"
    components = ('V',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class LocateAnimalsOrPlants(Spell):
    """Describe or name a specific kind of beast or plant. Concentrating on the voice 
    of nature in your surroundings, you learn the direction and distance to the 
    closest creature or plant of that kind within 5 miles, if any are present.
    """
    name = "Locate Animals Or Plants"
    level = 2
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A bit of fur from a bloodhound"""
    duration = "Instantaneous"
    ritual = True
    magic_school = "Divination"
    classes = ('Bard', 'Druid', 'Ranger')


class LocateCreature(Spell):
    """Describe or name a creature that is familiar to you. You sense the direction to 
    the creature’s location, as long as that creature is within 1,000 feet of you. 
    If the creature is moving, you know the direction of its movement.
    
    The spell 
    can locate a specific creature known to you, or the nearest creature of a 
    specific kind (such as a human or a unicorn), so long as you have seen such a 
    creature up close – within 30 feet – at least once. If the creature you 
    described or named is in a different form, such as being under the effects of a 
    polymorph spell, this spell doesn’t locate the creature.
    
    This spell can’t 
    locate a creature if running water at least 10 feet wide blocks a direct path 
    between you and the creature.
    """
    name = "Locate Creature"
    level = 4
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A bit of fur from a bloodhound"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Druid', 'Paladin', 'Ranger', 'Wizard')


class LocateObject(Spell):
    """Describe or name an object that is familiar to you. You sense the direction to 
    the object’s location, as long as that object is within 1,000 feet of you. If 
    the object is in motion, you know the direction of its movement.
    
    The spell can 
    locate a specific object known to you, as long as you have seen it up close – 
    within 30 feet – at least once. Alternatively, the spell can locate the nearest 
    object of a particular kind, such as a certain kind of apparel, jewelry, 
    furniture, tool, or weapon.
    
    This spell can’t locate an object if any thickness 
    of lead, even a thin sheet, blocks a direct path between you and the object.
    """
    name = "Locate Object"
    level = 2
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A forked twig"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Druid', 'Paladin', 'Ranger', 'Wizard')


class Longstrider(Spell):
    """You touch a creature. The target’s speed increases by 10 feet until the spell 
    ends.
    
    At Higher Levels: When you cast this spell using a spell slot of 2nd 
    level or higher, you can target one additional creature for each slot level 
    above 1st.
    """
    name = "Longstrider"
    level = 1
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """A pinch of dirt"""
    duration = "1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Bard', 'Druid', 'Ranger', 'Wizard')





from .features import Feature, FeatureSelector
from .. import spells


# PHB
class WildShape(Feature):
    """Starting at 2nd level, you can use your action to magically assume the
    shape of a beast that you have seen before. You can use this feature
    twice. You regain expended uses when you finish a short or long rest. Your
    druid level determines the beasts you can transform into, as shown in the
    Beast Shapes table. At 2nd level, for example, you can transform into any
    beast that has a challenge rating of 1/4 or lower that doesn’t have a
    flying or swimming speed.

    2nd Level: Max CR 1/4, No Flying/Swimming (ex: Wolf)
    
    4th Level: Max CR 1/2, No Flying (ex: Crocodile)

    8th Level: Max CR 1 (ex: Giant Eagle)

    You can stay in a beast shape for a number of hours equal to half your
    druid level (rounded down). You then revert to your normal form unless you
    expend another use of this feature. You can revert to your normal form
    earlier by using a bonus action on your turn. You automatically revert if
    you fall unconscious, drop to 0 hit points, or die.

    While you are transformed, the following rules apply:

    -- Your game statistics are replaced by the statistics of the beast, but
    you retain your alignment, personality, and Intelligence, Wisdom, and
    Charisma scores. You also retain all of your skill and saving throw
    proficiencies, in addition to gaining those of the creature. If the
    creature has the same proficiency as you and the bonus in its stat block is
    higher than yours, use the creature’s bonus instead of yours. If the
    creature has any legendary or lair actions, you can't use them.

    -- When you transform, you assume the beast’s hit points and Hit Dice. When
    you revert to your normal form, you return to the number of hit points you
    had before you transformed. However, if you revert as a result of dropping
    to 0 hit points, any excess damage carries over to your normal form. For
    example, if you take 10 damage in animal form and have only 1 hit point
    left, you revert and take 9 damage. As long as the excess damage doesn’t
    reduce your normal form to 0 hit points, you aren’t knocked unconscious.

    -- You can’t cast spells, and your ability to speak or take any action that
    requires hands is limited to the capabilities of your beast
    form. Transforming doesn’t break your concentration on a spell you’ve
    already cast, however, or prevent you from taking actions that are part of
    a spell, such as call lightning, that you’ve already cast.

    -- You retain the benefit of any features from your class, race, or other
    source and can use them if the new form is physically capable of doing
    so. However, you can’t use any of your special senses, such as darkvi-
    sion, unless your new form also has that sense.

    -- You choose whether your equipment falls to the ground in your space,
    merges into your new form, or is worn by it. Worn equipment functions as
    normal, but the DM decides whether it is practical for the new form to wear
    a piece of equipment, based on the creature’s shape and size. Your
    equipment doesn’t change size or shape to match the new form, and any
    equipment that the new form can’t wear must either fall to the ground or
    merge with it. Equipment that merges with the form has no effect until you
    leave the form.

    """
    _name = "Wild Shape"
    source = "Druid"

    @property
    def name(self):
        num = 2
        time = self.owner.Druid.level // 2
        return self._name + " ({:d}x/SR, {:d} hours)".format(num, time)


class TimelessBody(Feature):
    """Starting at 18th level, the primal magic that you wield causes you to age
    more slowly. For every 10 years that pass, your body ages only 1 year.

    """
    name = "Timeless Body"
    source = "Druid"


class BeastSpells(Feature):
    """Beginning at 18th level, you can cast many of your druid spells in any
    shape you assume using Wild Shape. You can perform the somatic and verbal
    components of a druid spell while in a beast shape, but you aren’t able to
    provide material components.

    """
    name = "Beast Spells"
    source = "Druid"

    
class Archdruid(Feature):
    """At 20th level, you can use your Wild Shape an unlimited number of times.

    Additionally, you can ignore the verbal and somatic components of your
    druid spells, as well as any material components that lack a cost and
    aren’t consumed by a spell. You gain this benefit in both your normal shape
    and your beast shape from Wild Shape

    """
    name = "Archdruid"
    source = "Druid"


# Circle of the Land
class BonusCantrip(Feature):
    """When you choose this circle at 2nd level, you learn one additional druid
    cantrip of your choice

    """
    name = "Bonus Cantrip"
    source = "Druid (Circle of the Land)"


class NaturalRecovery(Feature):
    """Starting at 2nd level, you can regain some of your magical energy by
    sitting in meditation and communing with nature. During a short rest, you
    choose expended spell slots to recover. The spell slots can have a combined
    level that is equal to or less than half your druid level (rounded up), and
    none of the slots can be 6th level or higher. You can’t use this feature
    again until you finish a long rest.

    For example, when you are a 4th-level druid, you can recover up to two
    levels worth of spell slots. You can recover either a 2nd-level slot or two
    1st-level slots.

    """
    name = "Natural Recovery"
    source = "Druid (Circle of the Land)"


class _CircleSpells(Feature):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells. At 3rd, 5th, 7th, and 9th level you gain access to circle
    spells connected to the land where you became a druid. Choose that
    land—arctic, coast, desert, forest, grassland, mountain, swamp, or
    Underdark—and consult the associated list of spells.

    Once you gain access to a circle spell, you always have it prepared, and it
    doesn’t count against the number of spells you can prepare each day. If you
    gain access to a spell that doesn’t appear on the druid spell list, the
    spell is nonetheless a druid spell for you.

    """
    _name = "Select One"
    source = "Druid (Circle of the Land)"
    _spells = {3: [spells.MirrorImage, spells.MistyStep],
               5: [spells.WaterBreathing, spells.WaterWalk],
               7: [spells.ControlWater, spells.FreedomOfMovement],
               9: [spells.ConjureElemental, spells.Scrying]}
    spells_known = []
    spells_prepared = []

    @property
    def name(self):
        return "Circle Spells ({:s})".format(self._name)

    def __init__(self, owner=None):
        if owner is not None:
            level = owner.Druid.level
            for lvl, sps in self._spells.items():
                if level >= lvl:
                    self.spells_known.extend(sps)
                    self.spells_prepared.extend(sps)
        super().__init__(owner=owner)
    

class ArcticSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Arctic'
    _spells = {3: [spells.HoldPerson, spells.SpikeGrowth],
               5: [spells.SleetStorm, spells.Slow],
               7: [spells.FreedomOfMovement, spells.IceStorm],
               9: [spells.CommuneWithNature, spells.ConeOfCold]}


class CoastSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Coast'
    _spells = {3: [spells.MirrorImage, spells.MistyStep],
               5: [spells.WaterBreathing, spells.WaterWalk],
               7: [spells.ControlWater, spells.FreedomOfMovement],
               9: [spells.ConjureElemental, spells.Scrying]}


class DesertSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Desert'
    _spells = {3: [spells.Blur, spells.Silence],
               5: [spells.CreateFoodAndWater, spells.ProtectionFromEnergy],
               7: [spells.Blight, spells.HallucinatoryTerrain],
               9: [spells.InsectPlague, spells.WallOfStone]}


class ForestSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Forest'
    _spells = {3: [spells.Barkskin, spells.SpiderClimb],
               5: [spells.CallLightning, spells.PlantGrowth],
               7: [spells.Divination, spells.FreedomOfMovement],
               9: [spells.CommuneWithNature, spells.TreeStride]}


class GrasslandSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Grassland'
    _spells = {3: [spells.Invisibility, spells.PassWithoutTrace],
               5: [spells.Daylight, spells.Haste],
               7: [spells.Divination, spells.FreedomOfMovement],
               9: [spells.Dream, spells.InsectPlague]}


class MountainSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Mountain'
    _spells = {3: [spells.SpiderClimb, spells.SpikeGrowth],
               5: [spells.LightningBolt, spells.MeldIntoStone],
               7: [spells.StoneShape, spells.Stoneskin],
               9: [spells.Passwall, spells.WallOfStone]}


class SwampSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Swamp'
    _spells = {3: [spells.Darkness, spells.MelfsAcidArrow],
               5: [spells.WaterWalk, spells.StinkingCloud],
               7: [spells.FreedomOfMovement, spells.LocateCreature],
               9: [spells.InsectPlague, spells.Scrying]}


class UnderdarkSpells(_CircleSpells):
    """Your mystical connection to the land infuses you with the ability to cast
    certain spells.

    These spells are included in your Spell Sheet

    """
    _name = 'Underdark'
    _spells = {3: [spells.SpiderClimb, spells.Web],
               5: [spells.GaseousForm, spells.StinkingCloud],
               7: [spells.GreaterInvisibility, spells.StoneShape],
               9: [spells.Cloudkill, spells.InsectPlague]}


class CircleSpells(FeatureSelector, _CircleSpells):
    """
    Select a land where you became a druid in feature_choices in your .py file:

    arctic

    coast

    desert

    forest

    grassland

    mountain

    swamp

    underdark
    
    """
    options = {'arctic': ArcticSpells,
               'coast': CoastSpells,
               'desert': DesertSpells,
               'forest': ForestSpells,
               'grassland': GrasslandSpells,
               'mountain': MountainSpells,
               'swamp': SwampSpells,
               'underdark': UnderdarkSpells}
    name = "Circle Spells (Select One)"
    source = "Druid (Circle of the Land)"


class LandsStride(Feature):
    """Starting at 6th level, moving through nonmagical difficult terrain costs
    you no extra movement. You can also pass through nonmagical plants without
    being slowed by them and without taking damage from them if they have
    thorns, spines, or a similar hazard. In addition, you have advantage on
    saving throws against plants that are magically created or manipulated to
    impede movement, such those created by the entangle spell.

    """
    name = "Land's Stride"
    source = "Class (Many)"


class NaturesWard(Feature):
    """When you reach 10th level, you can’t be charmed or frightened by elementals
    or fey, and you are immune to poison and disease

    """
    name = "Nature's Ward"
    source = "Druid (Circle of the Land)"


class NaturesSanctuary(Feature):
    """When you reach 14th level, creatures of the natural world sense your
    connection to nature and become hesitant to attack you. When a beast or
    plant creature attacks you, that creature must make a Wisdom saving throw
    against your druid spell save DC. On a failed save, the creature must
    choose a different target, or the attack automatically misses. On a
    successful save, the creature is immune to this effect for 24 hours. The
    creature is aware of this effect before it makes its attack against you

    """
    name = "Nature's Sanctuary"
    source = "Druid (Circle of the Land)"


# Circle of the Moon
class CombatWildShape(Feature):
    """When you choose this circle at 2nd level, you gain the ability to use Wild
    Shape on your turn as a bonus action, rather than as an
    action. Additionally, while you are transformed by Wild Shape, you can use
    a bonus action to expend one spell slot to regain 1d8 hit points per level
    of the spell slot expended.

    """
    name = "Combat Wild Shape"
    source = "Druid (Circle of the Moon)"


class CircleForms(Feature):
    """The rites of your circle grant you the ability to transform into more
    dangerous animal forms. Starting at 2nd level, you can use your Wild Shape
    to transform into a beast with a challenge rating as high as 1 (you ignore
    the Max. CR column of the Beast Shapes table, but must abide by the other
    limitations there). Starting at 6th level, you can transform into a beast
    with a challenge rating as high as your druid level divided by 3, rounded
    down

    """
    _name = "Circle Forms"
    source = "Druid (Circle of the Moon)"

    @property
    def name(self):
        level = self.owner.Druid.level
        return self._name + ' (CR {:d})'.format(max(1, level//3))


class PrimalStrike(Feature):
    """Starting at 6th level, your attacks in beast form count as magical for the
    purpose of overcoming resistance and immunity to nonmagical attacks and
    damage.

    """
    name = "Primal Strike"
    source = "Druid (Circle of the Moon)"


class ElementalWildShape(Feature):
    """At 10th level, you can expend two uses of Wild Shape at the same time to
    transform into an air elemental, an earth elemental, a fire elemental, or a
    water elemental.

    """
    name = "Elemental Wild Shape"
    source = "Druid (Circle of the Moon)"


class ThousandForms(Feature):
    """By 14th level, you have learned to use magic to alter your physical form in
    more subtle ways. You can cast the alter self spell at will.

    """
    name = "Thousand Forms"
    source = "Druid (Circle of the Moon)"
    spells_known = (spells.AlterSelf,)
    spells_prepared = (spells.AlterSelf,)


# Circle of Dreams
class BalmOfTheSummerCourt(Feature):
    """At 2nd level, you become imbued with the blessings of the Summer Court. You
    are a font of energy that offers respite from injuries. You have a pool of
    fey energy represented by a number of C168 equal to your druid level. As
    a bonus action, you can choose one creature you can see within 120 feet
    ofyou and spend a number of those dice equal to halfyour druid level or
    less. Roll the spent dice and add them together. The target regains a
    number of hit points equal to the total. The target also gains 1 temporary
    hit point per die spent. You regain all expended dice when you finish a
    long rest.

    """
    _name = "Balm Of The Summer Court"
    source = "Druid (Circle of Dreams)"

    @property
    def name(self):
        return self._name + " ({:d}x d6)".format(self.owner.Druid.level)


class HearthOfMoonlightAndShadow(Feature):
    """At 6th level, home can be wherever you are. During a short or long rest,
    you can invoke the shadowy power of the Gleaming Court to help guard your
    respite. At the start of the rest, you touch a point in space, and an
    invisible, 30-foot—radius sphere of magic appears, centered on that
    point. Total cover blocks the sphere. 

    While within the sphere, you and your allies gain a +5 bonus to Dexterity
    (Stealth) and Wisdom (Perception) checks, and any light from open flames in
    the sphere (a campfire, torches, or the like) isn’t visible outside it. The
    sphere vanishes at the end of the rest or when you leave the sphere

    """
    name = "Hearth of Moonlight and Shadow"
    source = "Druid (Circle of the Moon)"


class HiddenPaths(Feature):
    """Starting at 10th level, you can use the hidden, magical pathways that some
    fey use to traverse space in the blink of an eye. As a bonus action on your
    turn, you can teleport up to 60 feet to an unoccupied space you can
    see.
    
    Alternatively, you can use your action to teleport one willing creature you
    touch up to 30 feet to an unoccupied space you can see. You can use this
    feature a number of times equal to your Wisdom modifier (minimum of once),
    and you regain all expended uses of it when you finish a long rest

    """
    name = "Hidden Paths"
    source = "Druid (Circle of the Moon)"

    
class WalkerInDreams(Feature):
    """At 14th level, the magic of the Feywild grants you the ability to travel
    mentally or physically through dreamlands. When you finish a short rest,
    you can cast one of the following spells, without expending a spell slot or
    requiring material components: dream (with you as the messenger),
    scrying, or teleportation circle.

    This use of teleportation circle is special. Rather than opening a portal
    to a permanent teleportation circle, it opens a portal to the last location
    where you finished a long rest on your current plane of existence. If you
    haven*t taken a long rest on your current plane, the spell fails but isn’t
    wasted. Once you use this feature, you can’t use it again until you finish
    a long rest.

    """
    name = "Walker in Dreams"
    source = "Druid (Circle of the Moon)"


# Circle of the Shepherd
class SpeechOfTheWoods(Feature):
    """At 2nd level, you gain the ability to converse with beasts and many fey. You
    learn to speak, read, and write Sylvan. In addition, beasts can
    understand your speech, and you gain the ability to decipher their noises
    and motions. Most beasts lack the intelligence to convey or understand
    sophisticated concepts, but a friendly beast could relay what it has seen
    or heard in the recent past. This ability doesn’t grant you friendship with
    beasts, though you can combine this ability with gifts to curry favor with
    them as you would with any nonplayer character.

    """
    name = "Speech of the Woods"
    source = "Druid (Circle of the Shepherd)"


class SpiritTotem(Feature):
    """Starting at 2nd level, you can call forth nature spirits to influence the
    world around you. As a bonus action, you can magically summon an
    incorporeal spirit to a point you can see within 60 feet of you. The spirit
    creates an aura in a 30-foot radius around that point. It counts as neither
    a creature nor an object, though it has the spectral appearance of the
    creature it. represents.

    As a bonus action, you can move the spirit up to 60 feet to a point you can
    see. The spirit persists for 1 minute or until you’re incapacitated. Once
    you use this feature, you can’t use it again until you finish a short or
    long rest. The effect of the spirit’s aura depends on the type of spirit
    you summon from the options below.

    **Bear Spirit**: The bear spirit grants you and your allies its might and
    endurance. Each creature ofyour choice in the aura when the spirit appears
    gains temporary hit points equal to 5 + your druid level. In addition, you
    and your allies gain advantage on Strength checks and Strength saving
    throws while in the aura.

    **Hawk Spirit**: The hawk spirit is a consummate hunter, aiding you and
    your allies with its keen sight. When a creature makes an attack roll
    against a target in the spirit’s aura, you can use your reaction to grant
    advantage to that attack roll. In addition, you and your allies have
    advantage on Wisdom (Perception) checks while in the aura

    **Unicorn Spirit**: The unicorn spirit lends its protection to those
    nearby. You and your allies gain advantage on all ability checks made to
    detect creatures in the spirit’s aura. In addition. if you cast a spell
    using a spell slot that restores hit points to any creature inside or
    outside the aura, each creature of your choice in the aura also regains hit
    points equal to your druid level.

    """
    name = "Spirit Totem"
    source = "Druid (Circle of the Shepherd)"


class MightySummoner(Feature):
    """Starting at 6th level, beasts and fey that you conjure are more resilient
    than normal. Any beast or fey summoned or created by a spell that you cast
    gains the. following benefits:
    
    -- The creature appears with more hit points than normal: 2 extra hit
    points per Hit Die it has.

    -- The damage from its natural weapons is considered magical for the
    purpose of overcoming immunity and resistance to nonmagical attacks and
    damage.

    """
    name = "Mighty Summoner"
    source = "Druid (Circle of the Shepherd)"


class GuardianSpirit(Feature):
    """Beginning at 10th level, your Spirit Totem safeguards the beasts and fey
    that you call forth with your magic. When a beast or fey that you summoned
    or created with a spell ends its turn in your Spirit Totem aura, that crea—
    ture regains a number of hit points equal to halfyour druid level.

    """
    name = "Guardian Spirit"
    source = "Druid (Circle of the Shepherd)"


class FaithfulSummons(Feature):
    """Starting at 14th level, the nature spirits you commune with protect you
    when you are the most defenseless. Ifyou are reduced to 0 hit points or are
    incapacitated against your will, you can immediately gain the benefits of
    conjure animals as if it were cast using a 9th-level spell slot. It summons
    four beasts of your choice that are challenge rating 2 or lower. The
    conjured beasts appear within 20 feet of you. If they receive no commands
    from you, they protect you from harm and attack your foes. The spell lasts
    for 1 hour, requiring no concentration, or until you dismiss it (no action
    required). Once you use this feature, you can’t use it again until you
    finish a long rest

    """
    name = "Faithful Summons"
    source = "Druid (Circle of the Shepherd)"





from .spells import Spell


class TashasHideousLaughter(Spell):
    """A creature of your choice that you can see within range perceives everything as 
    hilariously funny and falls into fits of laughter if this spell affects it. The 
    target must succeed on a Wisdom saving throw or fall prone, becoming 
    incapacitated and unable to stand up for the duration. A creature with an 
    Intelligence score of 4 or less isn’t affected.
    
    At the end of each of its 
    turns, and each time it takes damage, the target can make another Wisdom saving 
    throw. The target has advantage on the saving throw ifit’s triggered by damage. 
    On a success, the spell ends.
    """
    name = "Tashas Hideous Laughter"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """Tiny tarts and a feather that is waved in the air"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Wizard')


class Telekinesis(Spell):
    """You gain the ability to move or manipulate creatures or objects by thought.
    When
     you cast the spell, and as your action each round for the duration, you can 
    exert your will on one creature or object that you can see within range, causing
     the appropriate effect below. You can affect the same target round after round,
     or choose a new one at any time. If you switch targets, the prior target is no 
    longer affected by the spell.
    
    Creature
    You can try to move a Huge or smaller 
    creature. Make an ability check with your spellcasting ability contested by the 
    creature’s Strength check. If you win the contest, you move the creature up to 
    30 feet in any direction, including upward but not beyond the range of this 
    spell. Until the end of your next turn, the creature is restrained in your 
    telekinetic grip. A creature lifted upward is suspended in mid-air.
    On 
    subsequent rounds, you can use your action to attempt to maintain your 
    telekinetic grip on the creature by repeating the contest.
    
    Object
    You can try 
    to move an object that weighs up to 1,000 pounds. If the object isn’t being worn
     or carried, you automatically move it up to 30 feet in any direction, but not 
    beyond the range of this spell.
    If the object is worn or carried by a creature, 
    you must make an ability check with your spellcasting ability contested by that 
    creature’s Strength check. If you succeed, you pull the object away from that 
    creature and can move it up to 30 feet in any direction but not beyond the range
     of this spell.
    You can exert fine control on objects with your telekinetic 
    grip, such as manipulating a simple tool, opening a door or a container, stowing
     or retrieving an item from an open container, or pouring the contents from a 
    vial.
    """
    name = "Telekinesis"
    level = 5
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Sorcerer', 'Wizard')


class Telepathy(Spell):
    """You create a telepathic link between yourself and a willing creature with which 
    you are familiar.
    The creature can be anywhere on the same plane of existence as
     you. The spell ends if you or the target are no longer on the same plane.
    
    
    Until the spell ends, you and the target can instantaneously share words, 
    images, sounds, and other sensory messages with one another through the link, 
    and the target recognizes you as the creature it is communicating with. The 
    spell enables a creature with an Intelligence score of at least 1 to understand 
    the meaning of your words and take in the scope of any sensory messages you send
     to it.
    """
    name = "Telepathy"
    level = 8
    casting_time = "1 action"
    casting_range = "Unlimited"
    components = ('V', 'S', 'M')
    materials = """A pair of linked silver rings"""
    duration = "24 hours"
    ritual = False
    magic_school = "Evocation"
    classes = ('Wizard',)


class Teleport(Spell):
    """This spell instantly transports you and up to eight willing creatures of your 
    choice that you can see within range, or a single object that you can see within
     range, to a destination you select. If you target an object, it must be able to
     fit entirely inside a 10-foot cube, and it can’t be held or carried by an 
    unwilling creature.
    
    The destination you choose must be known to you, and it 
    must be on the same plane of existence as you. Your familiarity with the 
    destination determines whether you arrive there successfully. The DM rolls d100 
    and consults the table.
    
    Familiarity 
        Mishap  Similar Area  Off Target  On Target
    Permanent circle   —       — 
            —     01-100
    Associated object    —       —       —       01-100
    Very 
    familiar     01-05      06-13     14-24    25-100
    Seen casually     01-33      
    34-43     44-53    54-100
    Viewed once    01-43      44-53     54-73    74-100
    
    Description       01-43      44-53     54-73    74-100
    False destination  01-50 
          51-100      —      —
    
    Familiarity.
    "Permanent circle" means a permanent 
    teleportation circle whose sigil sequence you know. "Associated object" means 
    that you possess an object taken from the desired destination within the last 
    six months, such as a book from a wizard's library, bed linen from a royal 
    suite, or a chunk of marble from a lich's secret tomb.
    "Very familiar" is a 
    place you have been very often, a place you have carefully studied, or a place 
    you can see when you cast the spell. "Seen casually" is someplace you have seen 
    more than once but with which you aren't very familiar. "Viewed once" is a place
     you have seen once, possibly using magic. "Description" is a place whose 
    location and appearance you know through someone else's description, perhaps 
    from a map.
    "False destination" is a place that doesn't exist. Perhaps you tried
     to scry an enemy's sanctum but instead viewed an illusion, or you are 
    attempting to teleport to a familiar location that no longer exists.
    
    On Target
    
    You and your group (or the target object) appear where you want to.
    
    Off Target
    
    You and your group (or the target object) appear a random distance away from the
     destination in a random direction. Distance off target is 1d10 x 1d10 percent 
    of the distance that was to be traveled. For example, if you tried to travel 120
     miles, landed off target, and rolled a 5 and 3 on the two d10s, then you would 
    be off target by 15 percent, or 18 miles. The DM determines the direction off 
    target randomly by rolling a d8 and designating 1 as north, 2 as northeast, 3 as
     east, and so on around the points of the compass. If you were teleporting to a 
    coastal city and wound up 18 miles out at sea, you could be in trouble.
    
    Similar
     Area
    You and your group (or the target object) wind up in a different area 
    that's visually or thematically similar to the target area. If you are heading 
    for your home laboratory, for example, you might wind up in another wizard's 
    laboratory or in an alchemical supply shop that has many of the same tools and 
    implements as your laboratory. Generally, you appear in the closest similar 
    place, but since the spell has no range limit, you could conceivably wind up 
    anywhere on the plane.
    
    Mishap
    The spell's unpredictable magic results in a 
    difficult journey. Each teleporting creature (or the target object) takes 3d10 
    force damage, and the DM rerolls on the table to see where you wind up (multiple
     mishaps can occur, dealing damage each time).
    """
    name = "Teleport"
    level = 7
    casting_time = "1 action"
    casting_range = "10 feet"
    components = ('V',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Bard', 'Sorcerer', 'Wizard')


class TeleportationCircle(Spell):
    """As you cast the spell, you draw a 10-foot-diameter circle on the ground 
    inscribed with sigils that link your location to a permanent teleportation 
    circle of your choice whose sigil sequence you know and that is on the same 
    plane of existence as you.
    A shimmering portal opens within the circle you drew 
    and remains open until the end of your next turn. Any creature that enters the 
    portal instantly appears within 5 feet of the destination circle or in the 
    nearest unoccupied space if that space is occupied.
    
    Many major temples, guilds,
     and other important places have permanent teleportation circles inscribed 
    somewhere within their confines. Each such circle includes a unique sigil 
    sequence – a string of magical runes arranged in a particular pattern. When you 
    first gain the ability to cast this spell, you learn the sigil sequences for two
     destinations on the Material Plane, determined by the DM. You can learn 
    additional sigil sequences during your adventures. You can commit a new sigil 
    sequence to memory after studying it for 1 minute.
    
    You can create a permanent 
    teleportation circle by casting this spell in the same location every day for 
    one year. You need not use the circle to teleport when you cast the spell in 
    this way.
    """
    name = "Teleportation Circle"
    level = 5
    casting_time = "1 minute"
    casting_range = "10 feet"
    components = ('V', 'M')
    materials = """Rare chalks and inks infused with precious gems with 50 gp, which the spell consumes"""
    duration = "1 round"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Bard', 'Sorcerer', 'Wizard')


class TempleOfTheGods(Spell):
    """You cause a temple to shimmer into existence on ground you can see within range.
     The temple must fit within an unoccupied cube of space, up to 120 feet on each 
    side. The temple remains until the spell ends. It is dedicated to whatever god, 
    pantheon, or philosophy is represented by the holy symbol used in the casting.
    
    You make all decisions about the temple’s appearance. The interior is enclosed 
    by a floor, walls, and a roof, with one door granting access to the interior and
     as many windows as you wish. Only you and any creatures you designate when you 
    cast the spell can open or close the door.
    The temple’s interior is an open 
    space with an idol or altar at one end. You decide whether the temple is 
    illuminated and whether that illumination is bright light or dim light. The 
    smell of burning incense fills the air within, and the temperature is mild.
    The 
    temple opposes types of creatures you choose when you cast this spell. Choose 
    one or more of the following: celestials, elementals, fey, fiends, or undead. If
     a creature of the chosen type attempts to enter the temple, that creature must 
    make a Charisma saving throw. On a failed save, it can’t enter the temple for 24
     hours. Even if the creature can enter the temple, the magic there hinders it; 
    whenever it makes an attack roll, an ability check, or a saving throw inside the
     temple, it must roll a d4 and subtract the number rolled from the d20 roll.
    In 
    addition, the sensors created by divination spells can’t appear inside the 
    temple, and creatures within can’t be targeted by divination spells.
    Finally, 
    whenever any creature in the temple regains hit points from a spell of 1st level
     or higher, the creature regains additional hit points equal to your Wisdom 
    modifier (minimum 1 hit point).
    The temple is made from opaque magical force 
    that extends into the Ethereal Plane, thus blocking ethereal travel into the 
    temple’s interior. Nothing can physically pass through the temple’s exterior. It
     can’t be dispelled by dispel magic, and antimagic field has no effect on it. A 
    disintegrate spell destroys the temple instantly.
    Casting this spell on the same
     spot every day for a year makes this effect permanent.
    """
    name = "Temple Of The Gods"
    level = 7
    casting_time = "1 hour"
    casting_range = "120 feet"
    components = ('V', 'S', 'M')
    materials = """A holy symbol worth at least 5 gp"""
    duration = "24 hours"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Cleric',)


class TensersFloatingDisk(Spell):
    """This spell creates a circular, horizontal plane of force, 3 feet in diameter and
     1 inch thick, that floats 3 feet above the ground in an unoccupied space of 
    your choice that you can see within range.
    The disk remains for the duration, 
    and can hold up to 500 pounds. If more weight is placed on it, the spell ends, 
    and everything on the disk falls to the ground.
    
    The disk is immobile while you 
    are within 20 feet of it. If you move more than 20 feet away from it, the disk 
    follows you so that it remains within 20 feet of you. It can more across uneven 
    terrain, up or down stairs, slopes and the like, but it can’t cross an elevation
     change of 10 feet or more. For example, the disk can’t move across a 10-foot-
    deep pit, nor could it leave such a pit if it was created at the bottom.
    
    If you
     move more than 100 feet from the disk (typically because it can’t move around 
    an obstacle to follow you), the spell ends.
    """
    name = "Tensers Floating Disk"
    level = 1
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """A drop of mercury"""
    duration = "1 hour"
    ritual = True
    magic_school = "Conjuration"
    classes = ('Wizard',)


class TensersTransformation(Spell):
    """You endow yourself with endurance and martial prowess fueled by magic. Until the
     spell ends, you can’t cast spells, and you gain the following benefits:
    - You 
    gain 50 temporary hit points. If any of these remain when the spell ends, they 
    are lost.
    - You have advantage on attack rolls that you make with simple and 
    martial weapons.
    - When you hit a target with a weapon attack, that target takes
     an extra 2d12 force
    damage.
    - You have proficiency with all armor, shields, 
    simple weapons, and martial weapons.
    - You have proficiency in Strength and 
    Constitution saving throws.
    - You can attack twice, instead of once, when you 
    take the Attack action on your turn. You ignore this benefit if you already have
     a feature, like Extra Attack, that gives you extra attacks.
    
    Immediately after 
    the spell ends, you must succeed on a DC 15 Constitution saving throw or suffer 
    one level of exhaustion.
    """
    name = "Tensers Transformation"
    level = 6
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S', 'M')
    materials = """A few hairs from a bull"""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Wizard',)


class Thaumaturgy(Spell):
    """You manifest a minor wonder, a sign of supernatural power, within range. You 
    create one of the following magical effects within range:
    
    * Your voice booms up
     to three times as loud as normal for 1 minute.
    * You cause flames to flicker, 
    brighten, dim, or change color for 1 minute.
    * You cause harmless tremors in the
     ground for 1 minute.
    * You create an instantaneous sound that originates from a
     point of your choice within range, such as a rumble of thunder, the cry of a 
    raven, or ominous whispers.
    * You instantaneously cause an unlocked door or 
    window to fly open or slam shut.
    * You alter the appearance of your eyes for 1 
    minute.
    
    If you cast this spell multiple times, you can have up to three of its 
    1-minute effects active at a time, and you can dismiss such an effect as an 
    action.
    """
    name = "Thaumaturgy"
    level = 0
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V',)
    materials = """"""
    duration = "Up to 1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Cleric',)


class ThornWhip(Spell):
    """You create a long, vine-like whip covered in thorns that lashes out at your 
    command toward a creature in range. Make a melee spell attack against the 
    target. If the attack hits, the creature takes 1d6 piercing damage, and if the 
    creature is Large or smaller, you pull the creature up to 10 feet closer to you.
    
    
    At Higher Levels: This spell’s damage increases by 1d6 when you reach 5th 
    level (2d6), 11th level (3d6), and 17th level (4d6).
    """
    name = "Thorn Whip"
    level = 0
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """The stem of a plant with thorns"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Druid',)


class ThunderStep(Spell):
    """You teleport yourself to an unoccupied space you can see within range. 
    Immediately after you disappear, a thunderous boom sounds, and each creature 
    within 10 feet of the space you left must make a Constitution saving throw, 
    taking 3d10 thunder damage on a failed save, or half as much damage on a 
    successful one. The thunder can be heard from up to 300 feet away.
    You can bring
     along objects as long as their weight doesn’t exceed what you can carry. You 
    can also teleport one willing creature of your size or smaller who is carrying 
    gear up to its carrying capacity. The creature must be within 5 feet of you when
     you cast this spell, and there must be an unoccupied space within 5 feet of 
    your destination space for the creature to appear in; otherwise, the creature is
     left behind.
    
    At Higher Levels: When you cast this spell using a spell slot of 
    4th level or higher, the damage increases by 1d10 for each slot level above 3rd.
    """
    name = "Thunder Step"
    level = 3
    casting_time = "1 action"
    casting_range = "90 feet"
    components = ('V',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Sorcerer', 'Warlock', 'Wizard')


class Thunderclap(Spell):
    """You create a burst of thunderous sound, which can be heard 100 feet away. 
    Each 
    creature other than you within 5 feet of you must make a Constitution saving 
    throw. On a failed save, the creature takes 1d6 thunder damage. 
    The spell’s 
    damage increases by 1d6 when you reach 5th level (2d6), 11th level (3d6), and 
    17th level (4d6).
    """
    name = "Thunderclap"
    level = 0
    casting_time = "1 action"
    casting_range = "5 feet"
    components = ('S',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Bard', 'Sorcerer', 'Druid', 'Warlock', 'Wizard')


class ThunderousSmite(Spell):
    """The first time you hit with a melee weapon attack during this spell’s duration, 
    your weapon rings with thunder that is audible within 300 feet of you, and the 
    attack deals an extra 2d6 thunder damage to the target. Additionally, if the 
    target is a creature, it must succeed on a Strength saving throw or be pushed 10
     feet away from you and knocked prone.
    """
    name = "Thunderous Smite"
    level = 1
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ('Paladin',)


class Thunderwave(Spell):
    """A wave of thunderous force sweeps out from you.
    Each creature in a 15-foot cube 
    originating from you must make a Constitution saving throw. On a failed save, a 
    creature takes 2d8 thunder damage and is pushed 10 feet away from you. On a 
    successful save, the creature takes half as much damage and isn’t pushed.
    
    In 
    addition, unsecured objects that are completely within the area of effect are 
    automatically pushed 10 feet away from you by the spell’s effect, and the spell 
    emits a thunderous boom audible out to 300 feet.
    
    At Higher Levels: When you 
    cast this spell using a spell slot of 2nd level or higher, the damage increases 
    by 1d8 for each slot level above 1st.
    """
    name = "Thunderwave"
    level = 1
    casting_time = "1 action"
    casting_range = "Self (15-foot cube)"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Bard', 'Druid', 'Sorcerer', 'Wizard')


class TidalWave(Spell):
    """You conjure up a wave of water that crashes down on an area within range. The 
    area can be up to 30 feet long, up to 10 feet wide, and up to 10 feet tall. Each
     creature in that area must make a Dexterity saving throw. On a failed save, a 
    creature takes 4d8 bludgeoning damage and is knocked prone. On a successful 
    save, a creature takes half as much damage and isn’t knocked prone. The water 
    then spreads out across the ground in all directions, extinguishing unprotected 
    flames in its area and within 30 feet of it, and then it vanishes.
    """
    name = "Tidal Wave"
    level = 3
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S', 'M')
    materials = """A drop of water"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Wizard', 'Sorcerer')


class TimeStop(Spell):
    """You briefly stop the flow of time for everyone but yourself. No time passes for 
    other creatures, while you take 1d4 + 1 turns in a row, during which you can use
     actions and move as normal.
    
    This spell ends if one of the actions you use 
    during this period, or any effects that you create during this period, affects a
     creature other than you or an object being worn or carried by someone other 
    than you. In addition, the spell ends if you move to a place more than 1,000 
    feet from the location where you cast it.
    """
    name = "Time Stop"
    level = 9
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V',)
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Sorcerer', 'Wizard')


class TinyServant(Spell):
    """You touch one Tiny, nonmagical object that isn’t attached to another object or a
     surface and isn’t being carried by another creature. The target animates and 
    sprouts little arms and legs, becoming a creature under your control until the 
    spell ends or the creature drops to 0 hit points. See the stat block for its 
    statistics.
    As a bonus action, you can mentally command the creature if it is 
    within 120 feet of you. (If you control multiple creatures with this spell, you 
    can command any or all of them at the same time, issuing the same command to 
    each one.) You decide what action the creature will take and where it will move 
    during its next turn, or you can issue a simple, general command, such as to 
    fetch a key, stand watch, or stack some books. If you issue no commands, the 
    servant does nothing other than defend itself against hostile creatures. Once 
    given an order, the servant continues to follow that order until its task is 
    complete.
    When the creature drops to 0 hit points, it reverts to its original 
    form, and any remaining damage carries over to that form.
    
    At Higher Levels: 
    When you cast this spell using a spell slot of 4th level or higher, you can 
    animate two additional objects for each slot level above 3rd.
    """
    name = "Tiny Servant"
    level = 3
    casting_time = "1 minute"
    casting_range = "Touch"
    components = ('V', 'S')
    materials = """"""
    duration = "8 hours"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Wizard',)


class TollTheDead(Spell):
    """You point at one creature you can see within range, and the sound of a dolorous 
    bell fills the air around it for a moment. The target must succeed on a Wisdom 
    saving throw or take 1d8 necrotic damage. If the target is missing any of its 
    hit points, it instead takes 1d12 necrotic damage.
    The spell’s damage increases 
    by one die when you reach 5th level (2d8 or 2d12), 11th level (3d8 or 3d12), and
     17th level (4d8 or 4d12).
    """
    name = "Toll The Dead"
    level = 0
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Warlock', 'Wizard', 'Cleric')


class Tongues(Spell):
    """This spell grants the creature you touch the ability to understand any spoken 
    language it hears. Moreover, when the target speaks, any creature that knows at 
    least one language and can hear the target understands what it says.
    """
    name = "Tongues"
    level = 3
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'M')
    materials = """A small clay model of a ziggurat"""
    duration = "1 hour"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Sorcerer', 'Warlock', 'Wizard')


class TransmuteRock(Spell):
    """You choose an area of stone or mud that you can see that fits within a 40-foot 
    cube and is within range, and choose one of the following effects.
    Transmute 
    Rock to Mud. Nonmagical rock of any sort in the area becomes an equal volume of 
    thick, flowing mud that remains for the spell’s duration.
    The ground in the 
    spell’s area becomes muddy enough that creatures can sink into it. Each foot 
    that a creature moves through the mud costs 4 feet of movement, and any creature
     on the ground when you cast the spell must make a Strength saving throw. A 
    creature must also make the saving throw when it moves into the area for the 
    first time on a turn or ends its turn there. On a failed save, a creature sinks 
    into the mud and is restrained, though it can use an action to end the 
    restrained condition on itself by pulling itself free of the mud.
    If you cast 
    the spell on a ceiling, the mud falls. Any creature under the mud when it falls 
    must make a Dexterity saving throw. A creature takes 4d8 bludgeoning damage on a
     failed save, or half as much damage on a successful one.
    Transmute Mud to Rock.
     Nonmagical mud or quicksand in the area no more than 10 feet deep transforms 
    into soft stone for the spell’s duration. Any creature in the mud when it 
    transforms must make a Dexterity saving throw. On a successful save, a creature 
    is shunted safely to the surface in an unoccupied space. On a failed save, a 
    creature becomes restrained by the rock. A restrained creature, or another 
    creature within reach, can use an action to try to break the rock by succeeding 
    on a DC 20 Strength check or by dealing damage to it. The rock has AC 15 and 25 
    hit points, and it is immune to poison and psychic damage.
    """
    name = "Transmute Rock"
    level = 5
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ('V', 'S', 'M')
    materials = """Clay and water"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Druid', 'Wizard')


class TransportViaPlants(Spell):
    """This spell creates a magical link between a Large or larger inanimate plant 
    within range and another plant, at any distance, on the same plane of existence.
     You must have seen or touched the destination plant at least once before. For 
    the duration, any creature can step into the target plant and exit from the 
    destination plant by using 5 feet of movement.
    """
    name = "Transport Via Plants"
    level = 6
    casting_time = "1 action"
    casting_range = "10 feet"
    components = ('V', 'S')
    materials = """"""
    duration = "1 round"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid',)


class TreeStride(Spell):
    """You gain the ability to enter a tree and move from inside it to inside another 
    tree of the same kind within 500 feet.
    Both trees must be living and at least 
    the same size as you. You must use 5 feet of movement to enter a tree. You 
    instantly know the location of all other trees of the same kind within 500 feet 
    and, as part of the move used to enter the tree, can either pass into one of 
    those trees or step out of the tree you’re in. You appear in a spot of your 
    choice within 5 feet of the destination tree, using another 5 feet of movement. 
    If you have no movement left, you appear within 5 feet of the tree you entered.
    
    
    You can use this transportation ability once per round for the duration. You 
    must end each turn outside a tree.
    """
    name = "Tree Stride"
    level = 5
    casting_time = "1 action"
    casting_range = "Self"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid', 'Ranger')


class TruePolymorph(Spell):
    """Choose one creature or nonmagical object that you can see within range. You 
    transform the creature into a different creature, the creature into an object, 
    or the object into a creature (the object must be neither worn nor carried by 
    another creature). The transformation lasts for the duration, or until the 
    target drops to 0 hit points or dies. If you concentrate on this spell for the 
    full duration, the transformation becomes permanent.
    
    Shapechangers aren’t 
    affected by this spell. An unwilling creature can make a Wisdom saving throw, 
    and if it succeeds, it isn’t affected by this spell.
    
    Creature into Creature
    If 
    you turn a creature into another kind of creature, the new form can be any kind 
    you choose whose challenge rating is equal to or less than the target’s (or its 
    level, if the target doesn’t have a challenge rating). The target’s game 
    statistics, including mental ability scores, are replaced by the statistics of 
    the new form. It retains its alignment and personality.
    The target assumes the 
    hit points of its new form, and when it reverts to its normal form, the creature
     returns to the number of hit points it had before it transformed. If it reverts
     as a result of dropping to 0 hit points, any excess damage carries over to its 
    normal form. As long as the excess damage doesn’t reduce the creature’s normal 
    form to 0 hit points, it isn’t knocked unconscious.
    The creature is limited in 
    the actions it can perform by the nature of its new form, and it can’t speak, 
    cast spells, or take any other action that requires hands or speech unless its 
    new form is capable of such actions.
    The target’s gear melds into the new form. 
    The creature can’t activate, use, wield, or otherwise benefit from any of its 
    equipment.
    
    Object into Creature
    You can turn an object into any kind of 
    creature, as long as the creature’s size is no larger than the object’s size and
     the creature’s challenge rating is 9 or lower. The creature is friendly to you 
    and your companions. It acts on each of your turns. You decide what action it 
    takes and how it moves. The DM has the creature’s statistics and resolves all of
     its actions and movement.
    If the spell becomes permanent, you no longer control
     the creature. It might remain friendly to you, depending on how you have 
    treated it.
    
    Creature into Object
    If you turn a creature into an object, it 
    transforms along with whatever it is wearing and carrying into that form. The 
    creature’s statistics become those of the object, and the creature has no memory
     of time spent in this form, after the spell ends and it returns to its normal 
    form.
    
    This spell can’t affect a target that has 0 hit points.
    """
    name = "True Polymorph"
    level = 9
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """A drop of mercury, a dollop of gum arabic, and a wisp of smoke"""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ('Bard', 'Warlock', 'Wizard')


class TrueResurrection(Spell):
    """You touch a creature that has been dead for no longer than 200 years and that 
    died for any reason except old age. If the creature’s soul is free and willing, 
    the creature is restored to life with all its hit points.
    
    This spell closes all
     wounds, neutralizes any poison, cures all diseases, and lifts any curses 
    affecting the creature when it died. The spell replaces damaged or missing 
    organs or limbs.
    
    The spell can even provide a new body if the original no 
    longer exists, in which case you must speak the creature’s name. The creature 
    then appears in an unoccupied space you choose within 10 feet of you.
    """
    name = "True Resurrection"
    level = 9
    casting_time = "1 hour"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """A sprinkle of holy water and diamonds worth at least 25,000 gp, which the spell consumes"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ('Cleric', 'Druid')


class TrueSeeing(Spell):
    """This spell gives the willing creature you touch the ability to see things as 
    they actually are. For the duration, the creature has truesight,  notices secret
     doors hidden by magic, and can see into the Ethereal Plane, all out to a range 
    of 120 feet.
    """
    name = "True Seeing"
    level = 6
    casting_time = "1 action"
    casting_range = "Touch"
    components = ('V', 'S', 'M')
    materials = """An ointment for the eyes that costs 25 gp; is made from mushroom powder, saffron, and fat; and is consumed by the spell"""
    duration = "1 hour"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Cleric', 'Sorcerer', 'Warlock', 'Wizard')


class TrueStrike(Spell):
    """You extend your hand and point a finger at a target in range. Your magic grants 
    you a brief insight into the target’s defenses. On your next turn, you gain 
    advantage on your first attack roll against the target, provided that this spell
     hasn’t ended.
    """
    name = "True Strike"
    level = 0
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('S',)
    materials = """"""
    duration = "Concentration, up to 1 round"
    ritual = False
    magic_school = "Divination"
    classes = ('Bard', 'Sorcerer', 'Warlock', 'Wizard')


class Tsunami(Spell):
    """A wall of water springs into existence at a point you choose within range. You 
    can make the wall up to 300 feet long, 300 feet high, and 50 feet thick. The 
    wall lasts for the duration.
    
    When the wall appears, each creature within its 
    area must make a Strength saving throw. On a failed save, a creature takes 6d10 
    bludgeoning damage, or half as much damage on a successful save.
    
    At the start 
    of each of your turns after the wall appears, the wall, along with any creatures
     in it, moves 50 feet away from you. Any Huge or smaller creature inside the 
    wall or whose space the wall enters when it moves must succeed on a Strength 
    saving throw or take 5d10 bludgeoning damage. A creature can take this damage 
    only once per round. At the end of the turn, the wall’s height is reduced by 50 
    feet, and the damage creatures take from the spell on subsequent rounds is 
    reduced by 1d10. When the wall reaches 0 feet in height, the spell ends.
    
    A 
    creature caught in the wall can move by swimming. Because of the force of the 
    wave, though, the creature must make a successful Strength (Athletics) check 
    against your spell save DC in order to move at all. If it fails the check, it 
    can’t move. A creature that moves out of the area falls to the ground.
    """
    name = "Tsunami"
    level = 8
    casting_time = "1 minute"
    casting_range = "Sight"
    components = ('V', 'S')
    materials = """"""
    duration = "Concentration, up to 6 rounds"
    ritual = False
    magic_school = "Conjuration"
    classes = ('Druid',)





from .spells import Spell


class OtilukesFreezingSphere(Spell):
    """A frigid globe of cold energy streaks from your fingertips to a point of your 
    choice within range, where it explodes in a 60-foot-radius sphere.
    Each creature
     within the area must make a Constitution saving throw. On a failed save, a 
    creature takes 10d6 cold damage. On a successful save, it takes half as much 
    damage.
    
    If the globe strikes a body of water or a liquid that is principally 
    water (not including water-based creatures), it freezes the liquid to a depth of
     6 inches over an area 30 feet square. This ice lasts for 1 minute. Creatures 
    that were swimming on the surface of frozen water are trapped in the ice. A 
    trapped creature can use an action to make a Strength check against your spell 
    save DC to break free.
    
    You can refrain from firing the globe after completing 
    the spell, if you wish. A small globe about the size of a sling stone, cool to 
    the touch, appears in your hand. At any time, you or a creature you give the 
    globe to can throw the globe (to a range of 40 feet) or hurl it with a sling (to
     the sling’s normal range). It shatters on impact, with the same effect as the 
    normal casting of the spell. You can also set the globe down without shattering 
    it. After 1 minute, if the globe hasn’t already shattered, it explodes.
    
    At 
    Higher Levels: When you cast this spell using a spell slot of 7th level or 
    higher, the damage increases by 1d6 for each slot level above 6th
    """
    name = "Otilukes Freezing Sphere"
    level = 6
    casting_time = "1 action"
    casting_range = "300 feet"
    components = ('V', 'S', 'M')
    materials = """A small crystal sphere"""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ('Wizard',)


class OtilukesResilientSphere(Spell):
    """A sphere of shimmering force encloses a creature or object of Large size or 
    smaller within range. An unwilling creature must make a Dexterity saving throw. 
    On a failed save, the creature is enclosed for the duration.
    
    Nothing---not 
    physical objects, energy, or other spell effects---can pass through the barrier,
     in or out, though a creature in the sphere can breathe there. The sphere is 
    immune to all damage, and a creature or object inside can’t be damaged by 
    attacks or effects originating from outside, nor can a creature inside the 
    sphere damage anything outside it.
    
    The sphere is weightless and just large 
    enough to contain the creature or object inside. An enclosed creature can use 
    its action to push against the sphere’s walls and thus roll the sphere at up to 
    half the creature’s speed. Similarly, the globe can be picked up and moved by 
    other creatures.
    
    A disintegrate spell targeting the globe destroys it without 
    harming anything inside it.
    """
    name = "Otilukes Resilient Sphere"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V', 'S', 'M')
    materials = """A hemispherical piece of clear crystal and a matching hemispherical piece of gum arabic"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ('Wizard',)


class OttosIrresistibleDance(Spell):
    """Choose one creature that you can see within range. The target begins a comic 
    dance in place: shuffling, tapping its feet, and capering for the duration. 
    Creatures that can’t be charmed are immune to this spell.
    
    A dancing creature 
    must use all its movement to dance without leaving its space and has 
    disadvantage on Dexterity saving throws and attack rolls. While the target is 
    affected by this spell, other creatures have advantage on attack rolls against 
    it. As an action, a dancing creature makes a Wisdom saving throw to regain 
    control of itself. On a successful save, the spell ends.
    """
    name = "Ottos Irresistible Dance"
    level = 6
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ('V',)
    materials = """"""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ('Bard', 'Wizard')


